@@ -7,6 +7,9 @@
  * you can also enter an input and output part of a pipeline
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <string.h>
 #include <gtk/gtk.h>
 #include <gst/gst.h>
@@ -183,8 +186,9 @@ ui_control_create (GstElement * element, GtkWidget * control, _filter_ui_t * ui)
       /* create the dparam object */
       dparam = gst_dpsmooth_new (G_PARAM_SPEC_VALUE_TYPE (specs[i]));
       g_object_set (G_OBJECT (dparam), "update_period", 2000000LL, NULL);
-      g_assert (gst_dpman_attach_dparam (dpman,
-              (gchar *) g_param_spec_get_name (specs[i]), dparam));
+      if (!gst_dpman_attach_dparam (dpman,
+              (gchar *) g_param_spec_get_name (specs[i]), dparam))
+        g_assert_not_reached ();
       gst_dpman_set_mode (dpman, "asynchronous");
       g_signal_connect (widget, "value-changed",
           G_CALLBACK (cb_dynparm_value_changed), dparam);
@@ -480,7 +484,7 @@ create_ui (_filter_ui_t * fui, _filter_data_t * fd)
   widget = gtk_label_new ("Output Pipe");
   gtk_container_add (GTK_CONTAINER (vbox), widget);
   fui->output = gtk_entry_new ();
-  gtk_entry_set_text (GTK_ENTRY (fui->output), "osssink fragment=1572872");     /* fixme: gconf default ? */
+  gtk_entry_set_text (GTK_ENTRY (fui->output), DEFAULT_AUDIOSINK);
   gtk_container_add (GTK_CONTAINER (vbox), fui->output);
   gtk_container_add (GTK_CONTAINER (fui->selection), vbox);
   g_signal_connect (G_OBJECT (fui->output), "activate",
