@@ -255,10 +255,14 @@ gst_alsa_sink_check_event (GstAlsaSink * sink, gint pad_nr)
       case GST_EVENT_FILLER:{
         guint64 dur = gst_event_filler_get_duration (event);
 
-        if (GST_CLOCK_TIME_IS_VALID (dur)) {
+        if (GST_CLOCK_TIME_IS_VALID (dur) && dur > 0) {
           gint samples;
           GstBuffer *buf;
 
+          /* No format negotiated yet, so pick a default one for this filler */
+          if (this->format == NULL && !gst_alsa_set_default_format (this))
+            break;
+
           buf = gst_buffer_new ();
           GST_BUFFER_TIMESTAMP (buf) = GST_EVENT_TIMESTAMP (event);
           GST_BUFFER_DURATION (buf) = dur;
