@@ -21,11 +21,14 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
+
 #include <string.h>
 
-#include "gstjpegdec.h"
+#include <gst/gst-i18n-plugin.h>
 #include <gst/video/video.h>
 
+#include "gstjpegdec.h"
+
 /* elementfactory information */
 GstElementDetails gst_jpegdec_details = {
   "JPEG image decoder",
@@ -37,26 +40,35 @@ GstElementDetails gst_jpegdec_details = {
 GST_DEBUG_CATEGORY (jpegdec_debug);
 #define GST_CAT_DEFAULT jpegdec_debug
 
-/* JpegDec signals and args */
-enum
-{
-  /* FILL ME */
-  LAST_SIGNAL
-};
 
-enum
-{
-  ARG_0
-      /* FILL ME */
-};
+/* These macros are adapted from videotestsrc.c 
+ *  and/or gst-plugins/gst/games/gstvideoimage.c */
+#define ROUND_UP_2(x)  (((x)+1)&~1)
+#define ROUND_UP_4(x)  (((x)+3)&~3)
+#define ROUND_UP_8(x)  (((x)+7)&~7)
+
+/* I420 */
+#define I420_Y_ROWSTRIDE(width) (ROUND_UP_4(width))
+#define I420_U_ROWSTRIDE(width) (ROUND_UP_8(width)/2)
+#define I420_V_ROWSTRIDE(width) (ROUND_UP_8(width)/2)
+
+#define I420_Y_OFFSET(w,h) (0)
+#define I420_U_OFFSET(w,h) (I420_Y_OFFSET(w,h)+(I420_Y_ROWSTRIDE(w)*ROUND_UP_2(h)))
+#define I420_V_OFFSET(w,h) (I420_U_OFFSET(w,h)+(I420_U_ROWSTRIDE(w)*ROUND_UP_2(h)/2))
+
+#define I420_SIZE(w,h)     (I420_V_OFFSET(w,h)+(I420_V_ROWSTRIDE(w)*ROUND_UP_2(h)/2))
+
 
 static void gst_jpegdec_base_init (gpointer g_class);
 static void gst_jpegdec_class_init (GstJpegDec * klass);
 static void gst_jpegdec_init (GstJpegDec * jpegdec);
+static void gst_jpegdec_finalize (GObject * object);
 
 static void gst_jpegdec_chain (GstPad * pad, GstData * _data);
 static GstPadLinkReturn gst_jpegdec_link (GstPad * pad, const GstCaps * caps);
 
+static GstElementStateReturn gst_jpegdec_change_state (GstElement * element);
+
 static GstElementClass *parent_class = NULL;
 
 /*static guint gst_jpegdec_signals[LAST_SIGNAL] = { 0 }; */
@@ -118,11 +130,17 @@ static void
 gst_jpegdec_class_init (GstJpegDec * klass)
 {
   GstElementClass *gstelement_class;
+  GObjectClass *gobject_class;
 
+  gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
 
   parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
 
+  gstelement_class->change_state = gst_jpegdec_change_state;
+
+  gobject_class->finalize = gst_jpegdec_finalize;
+
   GST_DEBUG_CATEGORY_INIT (jpegdec_debug, "jpegdec", 0, "JPEG decoder");
 }
 
@@ -158,6 +176,25 @@ gst_jpegdec_term_source (j_decompress_ptr cinfo)
   GST_DEBUG ("term_source");
 }
 
+METHODDEF (void)
+gst_jpegdec_my_output_message (j_common_ptr cinfo)
+{
+  // Do nothing
+}
+
+METHODDEF (void)
+gst_jpegdec_my_emit_message (j_common_ptr cinfo, int msg_level)
+{
+  // Do nothing
+}
+
+METHODDEF (void)
+gst_jpegdec_my_error_exit (j_common_ptr cinfo)
+{
+  (*cinfo->err->output_message) (cinfo);
+  longjmp (((struct GstJpegDecErrorMgr *) cinfo->err)->setjmp_buffer, 1);
+}
+
 static void
 gst_jpegdec_init (GstJpegDec * jpegdec)
 {
@@ -192,7 +229,11 @@ gst_jpegdec_init (GstJpegDec * jpegdec)
   /* setup jpeglib */
   memset (&jpegdec->cinfo, 0, sizeof (jpegdec->cinfo));
   memset (&jpegdec->jerr, 0, sizeof (jpegdec->jerr));
-  jpegdec->cinfo.err = jpeg_std_error (&jpegdec->jerr);
+  jpegdec->cinfo.err = jpeg_std_error (&jpegdec->jerr.pub);
+  jpegdec->jerr.pub.output_message = gst_jpegdec_my_output_message;
+  jpegdec->jerr.pub.emit_message = gst_jpegdec_my_emit_message;
+  jpegdec->jerr.pub.error_exit = gst_jpegdec_my_error_exit;
+
   jpeg_create_decompress (&jpegdec->cinfo);
 
   jpegdec->jsrc.init_source = gst_jpegdec_init_source;
@@ -204,6 +245,17 @@ gst_jpegdec_init (GstJpegDec * jpegdec)
 
 }
 
+static void
+gst_jpegdec_finalize (GObject * object)
+{
+
+  GstJpegDec *filter = GST_JPEGDEC (object);
+
+  jpeg_destroy_decompress (&filter->cinfo);
+
+  G_OBJECT_CLASS (parent_class)->finalize (object);
+}
+
 static GstPadLinkReturn
 gst_jpegdec_link (GstPad * pad, const GstCaps * caps)
 {
@@ -360,7 +412,7 @@ gst_jpegdec_chain (GstPad * pad, GstData * _data)
   GstBuffer *outbuf;
 
   /*GstMeta *meta; */
-  gint width, height, width2;
+  gint width, height;
   guchar *base[3];
   gint i, j, k;
   gint r_h, r_v;
@@ -384,6 +436,12 @@ gst_jpegdec_chain (GstPad * pad, GstData * _data)
   jpegdec->jsrc.next_input_byte = data;
   jpegdec->jsrc.bytes_in_buffer = size;
 
+  if (setjmp (jpegdec->jerr.setjmp_buffer)) {
+    GST_ELEMENT_ERROR (jpegdec, LIBRARY, TOO_LAZY,
+        (_("Failed to decode JPEG image")), (NULL));
+    gst_buffer_unref (buf);
+    return;
+  }
 
   GST_LOG_OBJECT (jpegdec, "reading header %08lx", *(gulong *) data);
   jpeg_read_header (&jpegdec->cinfo, TRUE);
@@ -428,7 +486,7 @@ gst_jpegdec_chain (GstPad * pad, GstData * _data)
   /* FIXME: someone needs to do the work to figure out how to correctly
    * calculate an output size that takes into account everything libjpeg
    * needs, like padding for DCT size and so on.  */
-  outsize = width * height + width * height / 2;
+  outsize = I420_SIZE (width, height);
   outbuf = gst_pad_alloc_buffer (jpegdec->srcpad, GST_BUFFER_OFFSET_NONE,
       outsize);
   outdata = GST_BUFFER_DATA (outbuf);
@@ -438,28 +496,25 @@ gst_jpegdec_chain (GstPad * pad, GstData * _data)
       height, outsize);
 
   /* mind the swap, jpeglib outputs blue chroma first */
-  /* FIXME: this needs stride love */
-  base[0] = outdata;
-  base[1] = base[0] + width * height;
-  base[2] = base[1] + width * height / 4;
-
-  width2 = width >> 1;
+  base[0] = outdata + I420_Y_OFFSET (width, height);
+  base[1] = outdata + I420_U_OFFSET (width, height);
+  base[2] = outdata + I420_V_OFFSET (width, height);
 
   GST_LOG_OBJECT (jpegdec, "decompressing %u",
       jpegdec->cinfo.rec_outbuf_height);
   for (i = 0; i < height; i += r_v * DCTSIZE) {
     for (j = 0, k = 0; j < (r_v * DCTSIZE); j += r_v, k++) {
       jpegdec->line[0][j] = base[0];
-      base[0] += width;
+      base[0] += I420_Y_ROWSTRIDE (width);
       if (r_v == 2) {
         jpegdec->line[0][j + 1] = base[0];
-        base[0] += width;
+        base[0] += I420_Y_ROWSTRIDE (width);
       }
       jpegdec->line[1][k] = base[1];
       jpegdec->line[2][k] = base[2];
       if (r_v == 2 || k & 1) {
-        base[1] += width2;
-        base[2] += width2;
+        base[1] += I420_U_ROWSTRIDE (width);
+        base[2] += I420_V_ROWSTRIDE (width);
       }
     }
     /*g_print ("%d\n", jpegdec->cinfo.output_scanline); */
@@ -474,3 +529,34 @@ gst_jpegdec_chain (GstPad * pad, GstData * _data)
 
   gst_buffer_unref (buf);
 }
+
+static GstElementStateReturn
+gst_jpegdec_change_state (GstElement * element)
+{
+
+  GstJpegDec *filter = GST_JPEGDEC (element);
+
+  switch (GST_STATE_TRANSITION (element)) {
+    case GST_STATE_NULL_TO_READY:
+      GST_DEBUG ("gst_jpegdec_change_state: setting line buffers");
+      filter->line[0] = NULL;
+      filter->line[1] = NULL;
+      filter->line[2] = NULL;
+      break;
+    case GST_STATE_READY_TO_NULL:
+      g_free (filter->line[0]);
+      g_free (filter->line[1]);
+      g_free (filter->line[2]);
+      filter->line[0] = NULL;
+      filter->line[1] = NULL;
+      filter->line[2] = NULL;
+      break;
+    default:
+      break;
+  }
+
+  if (GST_ELEMENT_CLASS (parent_class)->change_state)
+    return GST_ELEMENT_CLASS (parent_class)->change_state (element);
+
+  return GST_STATE_SUCCESS;
+}
