@@ -483,6 +483,8 @@ gst_vorbisenc_init (VorbisEnc * vorbisenc)
 
   vorbisenc->tags = gst_tag_list_new ();
 
+  vorbisenc->initial_delay = 0;
+
   /* we're chained and we can deal with events */
   GST_FLAG_SET (vorbisenc, GST_ELEMENT_EVENT_AWARE);
 }
@@ -711,21 +713,48 @@ gst_vorbisenc_setup (VorbisEnc * vorbisenc)
   return TRUE;
 }
 
-/* prepare a buffer for transmission by passing data through libvorbis */
+/* convert an ogg packet to a GstBuffer */
 static GstBuffer *
-gst_vorbisenc_buffer_from_packet (VorbisEnc * vorbisenc, ogg_packet * packet)
+gst_vorbisenc_buffer_from_packet (VorbisEnc * vorbisenc, ogg_packet * packet, gboolean isheader)
 {
   GstBuffer *outbuf;
+  guint64 granulepos_delta, timestamp_delta;
+
+  GST_DEBUG_OBJECT (vorbisenc,
+      "converting ogg packet with packetno %" G_GINT64_FORMAT
+      " and granulepos %" G_GINT64_FORMAT " to GstBuffer",
+      packet->packetno, packet->granulepos);
+  /* header packets always need to have granulepos 0,
+   * regardless of the initial delay */
+  if (isheader) {
+    GST_DEBUG_OBJECT (vorbisenc, "header packet, not adjusting granulepos");
+    granulepos_delta = 0;
+    timestamp_delta = 0;
+  } else {
+    granulepos_delta =
+        vorbisenc->initial_delay * vorbisenc->frequency / GST_SECOND;
+    timestamp_delta = vorbisenc->initial_delay;
+  }
 
   outbuf = gst_buffer_new_and_alloc (packet->bytes);
   memcpy (GST_BUFFER_DATA (outbuf), packet->packet, packet->bytes);
   GST_BUFFER_OFFSET (outbuf) = vorbisenc->bytes_out;
-  GST_BUFFER_OFFSET_END (outbuf) = packet->granulepos;
+
+  /* hack: for all codecs used by Ogg, we set the granulepos as
+   * OFFSET_END so the muxer can mux based on granulepos, as required */
+  GST_BUFFER_OFFSET_END (outbuf) = packet->granulepos + granulepos_delta;
   GST_BUFFER_TIMESTAMP (outbuf) =
       vorbis_granule_time_copy (&vorbisenc->vd,
-      packet->granulepos) * GST_SECOND;
+      packet->granulepos) * GST_SECOND + timestamp_delta;
+
+  GST_DEBUG ("encoded buffer of %d bytes. timestamp = %" GST_TIME_FORMAT
+      " granulepos = %" G_GINT64_FORMAT
+      " + delta %" G_GINT64_FORMAT " = %" G_GINT64_FORMAT,
+      GST_BUFFER_SIZE (outbuf),
+      GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (outbuf)),
+      packet->granulepos, granulepos_delta,
+      packet->granulepos + granulepos_delta);
 
-  GST_DEBUG ("encoded buffer of %d bytes", GST_BUFFER_SIZE (outbuf));
   return outbuf;
 }
 
@@ -742,12 +771,13 @@ gst_vorbisenc_push_buffer (VorbisEnc * vorbisenc, GstBuffer * buffer)
   }
 }
 
+/* push a regular non-header ogg packet out as a GstBuffer */
 static void
 gst_vorbisenc_push_packet (VorbisEnc * vorbisenc, ogg_packet * packet)
 {
   GstBuffer *outbuf;
 
-  outbuf = gst_vorbisenc_buffer_from_packet (vorbisenc, packet);
+  outbuf = gst_vorbisenc_buffer_from_packet (vorbisenc, packet, FALSE);
   gst_vorbisenc_push_buffer (vorbisenc, outbuf);
 }
 
@@ -816,6 +846,24 @@ gst_vorbisenc_chain (GstPad * pad, GstData * _data)
         }
         gst_pad_event_default (pad, event);
         return;
+      case GST_EVENT_DISCONTINUOUS:
+      {
+        gint64 val;
+
+        if (gst_event_discont_get_value (event, GST_FORMAT_TIME, &val)) {
+          /* vorbis does not support discontinuities in the middle of
+           * a stream so we can't just increase the granulepos to reflect
+           * the new position, or can we? would that still be according
+           * to spec? */
+          if (vorbisenc->samples_in == 0) {
+            vorbisenc->initial_delay = val;
+            GST_DEBUG ("initial delay = %" G_GUINT64_FORMAT, val);
+          } else {
+            GST_DEBUG ("mid stream discont: val = %" G_GUINT64_FORMAT, val);
+          }
+        }
+      }
+        /* fall through */
       default:
         gst_pad_event_default (pad, event);
         return;
@@ -853,9 +901,9 @@ gst_vorbisenc_chain (GstPad * pad, GstData * _data)
           &header_comm, &header_code);
 
       /* create header buffers */
-      buf1 = gst_vorbisenc_buffer_from_packet (vorbisenc, &header);
-      buf2 = gst_vorbisenc_buffer_from_packet (vorbisenc, &header_comm);
-      buf3 = gst_vorbisenc_buffer_from_packet (vorbisenc, &header_code);
+      buf1 = gst_vorbisenc_buffer_from_packet (vorbisenc, &header, TRUE);
+      buf2 = gst_vorbisenc_buffer_from_packet (vorbisenc, &header_comm, TRUE);
+      buf3 = gst_vorbisenc_buffer_from_packet (vorbisenc, &header_code, TRUE);
 
       /* mark and put on caps */
       caps = gst_pad_get_caps (vorbisenc->srcpad);
@@ -1045,6 +1093,11 @@ gst_vorbisenc_change_state (GstElement * element)
     case GST_STATE_PAUSED_TO_READY:
       vorbisenc->setup = FALSE;
       vorbisenc->header_sent = FALSE;
+      vorbisenc->initial_delay = 0;
+      vorbisenc->samples_in = 0;
+      vorbisenc->bytes_out = 0;
+      vorbisenc->channels = -1;
+      vorbisenc->frequency = -1;
       gst_tag_list_free (vorbisenc->tags);
       vorbisenc->tags = gst_tag_list_new ();
       break;
