@@ -46,6 +46,13 @@ G_BEGIN_DECLS
  * microphone, etc.). Channels are then single streams
  * within a track. A mono stream has one channel, a stereo
  * stream has two, etc.
+ *
+ * Input tracks can have 'recording' enabled, which means
+ * that any input will be hearable into the speakers that
+ * are attached to the output. Mute is obvious. A track
+ * flagged as master is the master volume track on this
+ * mixer, which means that setting this track will change
+ * the hearable volume on any output.
  */
 
 typedef enum {
@@ -53,22 +60,28 @@ typedef enum {
   GST_MIXER_TRACK_OUTPUT = (1<<1),
   GST_MIXER_TRACK_MUTE   = (1<<2),
   GST_MIXER_TRACK_RECORD = (1<<3),
+  GST_MIXER_TRACK_MASTER = (1<<4),
+  GST_MIXER_TRACK_SOFTWARE = (1<<5)
 } GstMixerTrackFlags;
 
 #define GST_MIXER_TRACK_HAS_FLAG(channel, flag) \
   ((channel)->flags & flag)
 
-typedef struct _GstMixerTrack {
+typedef struct _GstMixerTrack GstMixerTrack;
+typedef struct _GstMixerTrackClass GstMixerTrackClass;
+
+struct _GstMixerTrack {
   GObject            parent;
 
   gchar             *label;
+  /* FIXME: flags should be guint32. Change in 0.9 */
   GstMixerTrackFlags flags;
   gint               num_channels,
 	             min_volume,
 	             max_volume;
-} GstMixerTrack;
+};
 
-typedef struct _GstMixerTrackClass {
+struct _GstMixerTrackClass {
   GObjectClass parent;
 
   /* signals */
@@ -80,7 +93,7 @@ typedef struct _GstMixerTrackClass {
 			   gint          *volumes);
 
   gpointer _gst_reserved[GST_PADDING];
-} GstMixerTrackClass;
+};
 
 GType		gst_mixer_track_get_type	(void);
 
