@@ -113,6 +113,13 @@ GST_STATIC_CAPS ( \
     "width = (int) 16, " \
     "depth = (int) [ 1, 16 ], " \
     "signed = (boolean) { true, false }; " \
+  "audio/x-raw-int, " \
+    "rate = (int) [ 1, MAX ], " \
+    "channels = (int) [ 1, 8 ], " \
+    "endianness = (int) { LITTLE_ENDIAN, BIG_ENDIAN }, " \
+    "width = (int) 24, " \
+    "depth = (int) [ 1, 24 ], " \
+    "signed = (boolean) { true, false }; " \
   "audio/x-raw-int, " \
     "rate = (int) [ 1, MAX ], " \
     "channels = (int) [ 1, 8 ], " \
@@ -449,6 +456,7 @@ gst_audio_convert_link (GstPad * pad, const GstCaps * caps)
   }
 
   GST_DEBUG_OBJECT (this, "negotiated pad to %" GST_PTR_FORMAT, caps);
+  gst_audio_convert_setup_matrix (this);
   return GST_PAD_LINK_OK;
 }
 
@@ -523,9 +531,10 @@ gst_audio_convert_fixate (GstPad * pad, const GstCaps * caps)
   }
 
   if (_fixate_caps_to_int (&copy, "channels", try.channels)) {
-    int n;
+    int n, c;
 
-    if (try.channels > 2) {
+    gst_structure_get_int (gst_caps_get_structure (copy, 0), "channels", &c);
+    if (c > 2) {
       /* make sure we have a channelpositions structure or array here */
       GstStructure *str;
 
@@ -533,7 +542,7 @@ gst_audio_convert_fixate (GstPad * pad, const GstCaps * caps)
         str = gst_caps_get_structure (copy, n);
         if (!gst_structure_get_value (str, "channel-positions")) {
           /* first try otherpad's positions, else anything */
-          if (ac_caps.pos != NULL) {
+          if (ac_caps.pos != NULL && c == ac_caps.channels) {
             gst_audio_set_channel_positions (str, ac_caps.pos);
           } else {
             gst_audio_set_structure_channel_positions_list (str,
@@ -705,6 +714,23 @@ gst_audio_convert_buffer_to_default_format (GstAudioConvert * this,
                 this->sinkcaps.endianness, GUINT16_FROM_LE, GUINT16_FROM_BE);
           }
           break;
+        case 24:
+        {
+          /* Read 24-bits LE/BE into signed 64 host-endian */
+          if (this->sinkcaps.endianness == G_LITTLE_ENDIAN) {
+            cur = src[0] | (src[1] << 8) | (src[2] << 16);
+          } else {
+            cur = src[2] | (src[1] << 8) | (src[0] << 16);
+          }
+
+          /* Sign extend */
+          if ((this->sinkcaps.sign)
+              && (cur & (1 << (this->sinkcaps.depth - 1))))
+            cur |= ((gint64) (-1)) ^ ((1 << this->sinkcaps.depth) - 1);
+
+          src -= 3;
+        }
+          break;
         case 32:
           if (this->sinkcaps.sign) {
             CONVERT_TO (cur, src, gint32, this->sinkcaps.sign,
@@ -746,9 +772,9 @@ gst_audio_convert_buffer_to_default_format (GstAudioConvert * this,
   return ret;
 }
 
-#define POPULATE(format, be_func, le_func) G_STMT_START{			\
+#define POPULATE(out, format, be_func, le_func) G_STMT_START{			\
   format val;									\
-  format* p = (format *) dest;							\
+  format* p = (format *) out;							\
   int_value >>= (32 - this->srccaps.depth);					\
   if (this->srccaps.sign) {							\
     val = (format) int_value;							\
@@ -767,7 +793,7 @@ gst_audio_convert_buffer_to_default_format (GstAudioConvert * this,
   };                                                                           \
   *p = val;                                                                    \
   p ++;                                                                        \
-  dest = (guint8 *) p;                                                         \
+  out = (guint8 *) p;                                                          \
 }G_STMT_END
 
 static GstBuffer *
@@ -801,23 +827,45 @@ gst_audio_convert_buffer_from_default_format (GstAudioConvert * this,
       switch (this->srccaps.width) {
         case 8:
           if (this->srccaps.sign) {
-            POPULATE (gint8, GINT8_IDENTITY, GINT8_IDENTITY);
+            POPULATE (dest, gint8, GINT8_IDENTITY, GINT8_IDENTITY);
           } else {
-            POPULATE (guint8, GUINT8_IDENTITY, GUINT8_IDENTITY);
+            POPULATE (dest, guint8, GUINT8_IDENTITY, GUINT8_IDENTITY);
           }
           break;
         case 16:
           if (this->srccaps.sign) {
-            POPULATE (gint16, GINT16_TO_BE, GINT16_TO_LE);
+            POPULATE (dest, gint16, GINT16_TO_BE, GINT16_TO_LE);
           } else {
-            POPULATE (guint16, GUINT16_TO_BE, GUINT16_TO_LE);
+            POPULATE (dest, guint16, GUINT16_TO_BE, GUINT16_TO_LE);
           }
           break;
+        case 24:
+        {
+          guint8 tmp[4];
+          guint8 *tmpp = tmp;
+
+          /* Write out big endian array */
+          if (this->srccaps.sign) {
+            POPULATE (tmpp, gint32, GINT32_TO_BE, GINT32_TO_BE);
+          } else {
+            POPULATE (tmpp, guint32, GUINT32_TO_BE, GUINT32_TO_BE);
+          }
+
+          if (this->srccaps.endianness == G_LITTLE_ENDIAN) {
+            dest[2] = tmp[1];
+            dest[1] = tmp[2];
+            dest[0] = tmp[3];
+          } else {
+            memcpy (dest, tmp + 1, 3);
+          }
+          dest += 3;
+        }
+          break;
         case 32:
           if (this->srccaps.sign) {
-            POPULATE (gint32, GINT32_TO_BE, GINT32_TO_LE);
+            POPULATE (dest, gint32, GINT32_TO_BE, GINT32_TO_LE);
           } else {
-            POPULATE (guint32, GUINT32_TO_BE, GUINT32_TO_LE);
+            POPULATE (dest, guint32, GUINT32_TO_BE, GUINT32_TO_LE);
           }
           break;
         default:
@@ -853,10 +901,7 @@ gst_audio_convert_channels (GstAudioConvert * this, GstBuffer * buf)
   GstBuffer *ret;
   gint count;
 
-  /* setup if not yet done */
-  if (!this->matrix)
-    gst_audio_convert_setup_matrix (this);
-
+  g_assert (this->matrix != NULL);
   /* check for passthrough */
   if (gst_audio_convert_passthrough (this))
     return buf;
