@@ -25,6 +25,7 @@
 #include "gstplaybasebin.h"
 #include "gststreamselector.h"
 #include "gstplay-marshal.h"
+#include <gst/gst-i18n-plugin.h>
 
 GST_DEBUG_CATEGORY_STATIC (gst_play_base_bin_debug);
 #define GST_CAT_DEFAULT gst_play_base_bin_debug
@@ -63,7 +64,7 @@ enum
 
 static void gst_play_base_bin_class_init (GstPlayBaseBinClass * klass);
 static void gst_play_base_bin_init (GstPlayBaseBin * play_base_bin);
-static void gst_play_base_bin_dispose (GObject * object);
+static void gst_play_base_bin_finalize (GObject * object);
 
 static void gst_play_base_bin_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * spec);
@@ -221,7 +222,7 @@ gst_play_base_bin_class_init (GstPlayBaseBinClass * klass)
       G_STRUCT_OFFSET (GstPlayBaseBinClass, unlink_stream),
       NULL, NULL, gst_marshal_VOID__OBJECT, G_TYPE_NONE, 1, G_TYPE_OBJECT);
 
-  gobject_klass->dispose = GST_DEBUG_FUNCPTR (gst_play_base_bin_dispose);
+  gobject_klass->finalize = GST_DEBUG_FUNCPTR (gst_play_base_bin_finalize);
 
   /* we handle state changes like an element */
   gstelement_klass->set_state = GST_ELEMENT_CLASS (element_class)->set_state;
@@ -260,18 +261,20 @@ gst_play_base_bin_init (GstPlayBaseBin * play_base_bin)
 }
 
 static void
-gst_play_base_bin_dispose (GObject * object)
+gst_play_base_bin_finalize (GObject * object)
 {
   GstPlayBaseBin *play_base_bin;
 
   play_base_bin = GST_PLAY_BASE_BIN (object);
-  g_free (play_base_bin->uri);
-  play_base_bin->uri = NULL;
-  g_free (play_base_bin->suburi);
-  play_base_bin->suburi = NULL;
 
-  if (G_OBJECT_CLASS (parent_class)->dispose) {
-    G_OBJECT_CLASS (parent_class)->dispose (object);
+  g_free (play_base_bin->uri);
+  g_free (play_base_bin->suburi);
+
+  g_mutex_free (play_base_bin->group_lock);
+  g_cond_free (play_base_bin->group_cond);
+
+  if (G_OBJECT_CLASS (parent_class)->finalize) {
+    G_OBJECT_CLASS (parent_class)->finalize (object);
   }
 }
 
@@ -597,7 +600,8 @@ gen_preroll_element (GstPlayBaseBin * play_base_bin,
   name = g_strdup_printf ("preroll_%s_%s", prename, gst_pad_get_name (pad));
   preroll = gst_element_factory_make ("queue", name);
   g_object_set (G_OBJECT (preroll),
-      "max-size-buffers", 0, "max-size-bytes", 10 * 1024 * 1024,
+      "max-size-buffers", 0, "max-size-bytes",
+      (type == GST_STREAM_TYPE_VIDEO) ? 25 * 1024 * 1024 : 1024 * 1024,
       "max-size-time", play_base_bin->queue_size, NULL);
   sig = g_signal_connect (G_OBJECT (preroll), "overrun",
       G_CALLBACK (queue_overrun), play_base_bin);
@@ -1186,7 +1190,7 @@ setup_source (GstPlayBaseBin * play_base_bin,
   if (!play_base_bin->source) {
     /* whoops, could not create the source element */
     g_set_error (error, 0, 0,
-        "No URI handler implemented for \"%s\"", play_base_bin->uri);
+        _("No URI handler implemented for \"%s\""), play_base_bin->uri);
     GST_WARNING ("don't know how to read %s", play_base_bin->uri);
     play_base_bin->source = old_src;
     return FALSE;
@@ -1467,12 +1471,12 @@ prepare_output (GstPlayBaseBin * play_base_bin)
   if (!stream_found) {
     if (!no_media) {
       GST_ELEMENT_ERROR (play_base_bin, STREAM, CODEC_NOT_FOUND,
-          ("There were no decoders found to handle the stream in file "
-              "\"%s\", you might need to install the corresponding plugins",
-              play_base_bin->uri), (NULL));
+          (_("There were no decoders found to handle the stream, "
+                  "you might need to install the corresponding plugins")),
+          (NULL));
     } else {
       GST_ELEMENT_ERROR (play_base_bin, STREAM, WRONG_TYPE,
-          ("File \"%s\" is not a media file", play_base_bin->uri), (NULL));
+          (_("The stream is not a media file")), (NULL));
     }
     return FALSE;
   }
@@ -1771,13 +1775,11 @@ gst_play_base_bin_change_state (GstElement * element)
         if (!error) {
           /* opening failed but no error - hellup */
           GST_ELEMENT_ERROR (GST_ELEMENT (play_base_bin), STREAM,
-              NOT_IMPLEMENTED,
-              ("cannot open file \"%s\"", play_base_bin->uri), (NULL));
+              NOT_IMPLEMENTED, (NULL), (NULL));
         } else {
-          /* just copy the cached error - type doesn't matter */
-          GST_ELEMENT_ERROR (play_base_bin, STREAM, TOO_LAZY,
-              (error->message), (NULL));
-          g_error_free (error);
+          gst_element_error_full (GST_ELEMENT (play_base_bin),
+              error->domain, error->code, g_strdup (error->message),
+              NULL, __FILE__, GST_FUNCTION, __LINE__);
         }
         ret = GST_STATE_FAILURE;
       } else if (new_location) {
