@@ -85,12 +85,12 @@ enum
   LAST_SIGNAL
 };
 
-/* this is really arbitrary choosen */
+/* this is really arbitrarily chosen */
 #define DEFAULT_PROTOCOL		 GST_TCP_PROTOCOL_TYPE_NONE
 #define DEFAULT_MODE			 GST_FDSET_MODE_POLL
 #define DEFAULT_BUFFERS_MAX		-1
 #define DEFAULT_BUFFERS_SOFT_MAX	-1
-#define DEFAULT_UNIT_TYPE	 	GST_UNIT_TYPE_BUFFERS
+#define DEFAULT_UNIT_TYPE		GST_UNIT_TYPE_BUFFERS
 #define DEFAULT_UNITS_MAX		-1
 #define DEFAULT_UNITS_SOFT_MAX		-1
 #define DEFAULT_RECOVER_POLICY		 GST_RECOVER_POLICY_NONE
@@ -275,6 +275,9 @@ gst_multifdsink_class_init (GstMultiFdSinkClass * klass)
 
   parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
 
+  gobject_class->set_property = gst_multifdsink_set_property;
+  gobject_class->get_property = gst_multifdsink_get_property;
+
   g_object_class_install_property (gobject_class, ARG_PROTOCOL,
       g_param_spec_enum ("protocol", "Protocol", "The protocol to wrap data in",
           GST_TYPE_TCP_PROTOCOL_TYPE, DEFAULT_PROTOCOL, G_PARAM_READWRITE));
@@ -375,9 +378,6 @@ gst_multifdsink_class_init (GstMultiFdSinkClass * klass)
           client_removed), NULL, NULL, gst_tcp_marshal_VOID__INT_BOXED,
       G_TYPE_NONE, 2, G_TYPE_INT, GST_TYPE_CLIENT_STATUS);
 
-  gobject_class->set_property = gst_multifdsink_set_property;
-  gobject_class->get_property = gst_multifdsink_get_property;
-
   gstelement_class->change_state = gst_multifdsink_change_state;
 
   klass->add = gst_multifdsink_add;
@@ -401,7 +401,7 @@ gst_multifdsink_init (GstMultiFdSink * this)
   this->protocol = DEFAULT_PROTOCOL;
   this->mode = DEFAULT_MODE;
 
-  this->clientslock = g_mutex_new ();
+  CLIENTS_LOCK_INIT (this);
   this->clients = NULL;
   this->fd_hash = g_hash_table_new (g_int_hash, g_int_equal);
 
@@ -445,13 +445,13 @@ gst_multifdsink_add (GstMultiFdSink * sink, int fd)
   /* send last activity time to connect time */
   client->last_activity_time = GST_TIMEVAL_TO_TIME (now);
 
-  g_mutex_lock (sink->clientslock);
+  CLIENTS_LOCK (sink);
 
   /* check the hash to find a duplicate fd */
   clink = g_hash_table_lookup (sink->fd_hash, &client->fd.fd);
   if (clink != NULL) {
     client->status = GST_CLIENT_STATUS_DUPLICATE;
-    g_mutex_unlock (sink->clientslock);
+    CLIENTS_UNLOCK (sink);
     GST_WARNING_OBJECT (sink, "[fd %5d] duplicate client found, refusing", fd);
     g_signal_emit (G_OBJECT (sink),
         gst_multifdsink_signals[SIGNAL_CLIENT_REMOVED], 0, fd, client->status);
@@ -481,7 +481,7 @@ gst_multifdsink_add (GstMultiFdSink * sink, int fd)
 
   SEND_COMMAND (sink, CONTROL_RESTART);
 
-  g_mutex_unlock (sink->clientslock);
+  CLIENTS_UNLOCK (sink);
 
   g_signal_emit (G_OBJECT (sink),
       gst_multifdsink_signals[SIGNAL_CLIENT_ADDED], 0, fd);
@@ -494,7 +494,7 @@ gst_multifdsink_remove (GstMultiFdSink * sink, int fd)
 
   GST_DEBUG_OBJECT (sink, "[fd %5d] removing client", fd);
 
-  g_mutex_lock (sink->clientslock);
+  CLIENTS_LOCK (sink);
   clink = g_hash_table_lookup (sink->fd_hash, &fd);
   if (clink != NULL) {
     GstTCPClient *client = (GstTCPClient *) clink->data;
@@ -505,7 +505,7 @@ gst_multifdsink_remove (GstMultiFdSink * sink, int fd)
   } else {
     GST_WARNING_OBJECT (sink, "[fd %5d] no client with this fd found!", fd);
   }
-  g_mutex_unlock (sink->clientslock);
+  CLIENTS_UNLOCK (sink);
 }
 
 void
@@ -515,7 +515,7 @@ gst_multifdsink_clear (GstMultiFdSink * sink)
 
   GST_DEBUG_OBJECT (sink, "clearing all clients");
 
-  g_mutex_lock (sink->clientslock);
+  CLIENTS_LOCK (sink);
   for (clients = sink->clients; clients; clients = next) {
     GstTCPClient *client;
 
@@ -526,7 +526,7 @@ gst_multifdsink_clear (GstMultiFdSink * sink)
     gst_multifdsink_remove_client_link (sink, clients);
   }
   SEND_COMMAND (sink, CONTROL_RESTART);
-  g_mutex_unlock (sink->clientslock);
+  CLIENTS_UNLOCK (sink);
 }
 
 GValueArray *
@@ -536,7 +536,7 @@ gst_multifdsink_get_stats (GstMultiFdSink * sink, int fd)
   GValueArray *result = NULL;
   GList *clink;
 
-  g_mutex_lock (sink->clientslock);
+  CLIENTS_LOCK (sink);
   clink = g_hash_table_lookup (sink->fd_hash, &fd);
   client = (GstTCPClient *) clink->data;
   if (client != NULL) {
@@ -574,7 +574,7 @@ gst_multifdsink_get_stats (GstMultiFdSink * sink, int fd)
     g_value_set_uint64 (&value, client->last_activity_time);
     result = g_value_array_append (result, &value);
   }
-  g_mutex_unlock (sink->clientslock);
+  CLIENTS_UNLOCK (sink);
 
   /* python doesn't like a NULL pointer yet */
   if (result == NULL) {
@@ -639,22 +639,22 @@ gst_multifdsink_remove_client_link (GstMultiFdSink * sink, GList * link)
   g_slist_foreach (client->sending, (GFunc) gst_data_unref, NULL);
   g_slist_free (client->sending);
   client->sending = NULL;
-
-  /* unlock the mutex before signaling because the signal handler
-   * might query some properties */
-  g_mutex_unlock (sink->clientslock);
+  CLIENTS_UNLOCK (sink);
 
   g_signal_emit (G_OBJECT (sink),
       gst_multifdsink_signals[SIGNAL_CLIENT_REMOVED], 0, fd, client->status);
 
-  /* lock again before we remove the client completely */
-  g_mutex_lock (sink->clientslock);
-
+  CLIENTS_LOCK (sink);
   if (!g_hash_table_remove (sink->fd_hash, &client->fd.fd)) {
     GST_WARNING_OBJECT (sink,
         "[fd %5d] error removing client %p from hash", client->fd.fd, client);
   }
-  sink->clients = g_list_delete_link (sink->clients, link);
+  /* after releasing the lock above, the link could be invalid, more
+   * precisely, the next and prev pointers could point to invalid list
+   * links. One optimisation could be to add a cookie to the linked list
+   * and take a shortcut when it did not change between unlocking and locking
+   * our mutex. For now we just walk the list again. */
+  sink->clients = g_list_remove (sink->clients, client);
 
   if (fclass->removed)
     fclass->removed (sink, client->fd.fd);
@@ -737,7 +737,7 @@ gst_multifdsink_client_queue_data (GstMultiFdSink * sink, GstTCPClient * client,
   GstBuffer *buf;
 
   buf = gst_buffer_new ();
-  GST_BUFFER_DATA (buf) = data;
+  GST_BUFFER_DATA (buf) = (guint8 *) data;
   GST_BUFFER_SIZE (buf) = len;
 
   GST_LOG_OBJECT (sink, "[fd %5d] queueing data of length %d",
@@ -766,10 +766,10 @@ gst_multifdsink_client_queue_caps (GstMultiFdSink * sink, GstTCPClient * client,
     GST_DEBUG_OBJECT (sink, "Could not create GDP packet from caps");
     return FALSE;
   }
-  gst_multifdsink_client_queue_data (sink, client, header, length);
+  gst_multifdsink_client_queue_data (sink, client, (gchar *) header, length);
 
   length = gst_dp_header_payload_length (header);
-  gst_multifdsink_client_queue_data (sink, client, payload, length);
+  gst_multifdsink_client_queue_data (sink, client, (gchar *) payload, length);
 
   return TRUE;
 }
@@ -798,7 +798,7 @@ gst_multifdsink_client_queue_buffer (GstMultiFdSink * sink,
           "[fd %5d] could not create header, removing client", client->fd.fd);
       return FALSE;
     }
-    gst_multifdsink_client_queue_data (sink, client, header, len);
+    gst_multifdsink_client_queue_data (sink, client, (gchar *) header, len);
   }
 
   GST_LOG_OBJECT (sink, "[fd %5d] queueing buffer of length %d",
@@ -948,6 +948,8 @@ gst_multifdsink_handle_client_write (GstMultiFdSink * sink,
   /* if we have streamheader buffers, and haven't sent them to this client
    * yet, send them out one by one */
   if (!client->streamheader_sent) {
+    GST_DEBUG_OBJECT (sink, "[fd %5d] Sending streamheader, %d buffers", fd,
+        g_slist_length (sink->streamheader));
     if (sink->streamheader) {
       GSList *l;
 
@@ -1041,7 +1043,7 @@ gst_multifdsink_handle_client_write (GstMultiFdSink * sink,
           more = FALSE;
         } else if (errno == ECONNRESET) {
           GST_DEBUG_OBJECT (sink, "[fd %5d] connection reset by peer, removing",
-            fd);
+              fd);
           client->status = GST_CLIENT_STATUS_CLOSED;
           return FALSE;
         } else {
@@ -1160,7 +1162,7 @@ gst_multifdsink_queue_buffer (GstMultiFdSink * sink, GstBuffer * buf)
   g_get_current_time (&nowtv);
   now = GST_TIMEVAL_TO_TIME (nowtv);
 
-  g_mutex_lock (sink->clientslock);
+  CLIENTS_LOCK (sink);
   /* add buffer to queue */
   g_array_prepend_val (sink->bufqueue, buf);
   queuelen = sink->bufqueue->len;
@@ -1260,7 +1262,7 @@ gst_multifdsink_queue_buffer (GstMultiFdSink * sink, GstBuffer * buf)
   }
   /* save for stats */
   sink->buffers_queued = max_buffer_usage;
-  g_mutex_unlock (sink->clientslock);
+  CLIENTS_UNLOCK (sink);
 
   /* and send a signal to thread if fd_set changed */
   if (need_signal) {
@@ -1301,11 +1303,11 @@ gst_multifdsink_handle_clients (GstMultiFdSink * sink)
     /* < 0 is an error, 0 just means a timeout happened, which is impossible */
     if (result < 0) {
       GST_WARNING_OBJECT (sink, "wait failed: %s (%d)", g_strerror (errno),
-        errno);
+          errno);
       if (errno == EBADF) {
         /* ok, so one or more of the fds is invalid. We loop over them to find
          * the ones that give an error to the F_GETFL fcntl. */
-        g_mutex_lock (sink->clientslock);
+        CLIENTS_LOCK (sink);
         for (clients = sink->clients; clients; clients = next) {
           GstTCPClient *client;
           int fd;
@@ -1327,7 +1329,7 @@ gst_multifdsink_handle_clients (GstMultiFdSink * sink)
             }
           }
         }
-        g_mutex_unlock (sink->clientslock);
+        CLIENTS_UNLOCK (sink);
         /* after this, go back in the select loop as the read/writefds
          * are not valid */
         try_again = TRUE;
@@ -1389,7 +1391,7 @@ gst_multifdsink_handle_clients (GstMultiFdSink * sink)
     fclass->wait (sink, sink->fdset);
 
   /* Check the clients */
-  g_mutex_lock (sink->clientslock);
+  CLIENTS_LOCK (sink);
   for (clients = sink->clients; clients; clients = next) {
     GstTCPClient *client;
 
@@ -1427,7 +1429,7 @@ gst_multifdsink_handle_clients (GstMultiFdSink * sink)
       }
     }
   }
-  g_mutex_unlock (sink->clientslock);
+  CLIENTS_UNLOCK (sink);
 }
 
 /* we handle the client communication in another thread so that we do not block
@@ -1458,6 +1460,18 @@ gst_multifdsink_chain (GstPad * pad, GstData * _data)
     return;
   }
 
+  GST_LOG_OBJECT (sink, "received buffer %p", buf);
+  /* if we get IN_CAPS buffers, but the previous buffer was not IN_CAPS,
+   * it means we're getting new streamheader buffers, and we should clear
+   * the old ones */
+  if (GST_BUFFER_FLAG_IS_SET (buf, GST_BUFFER_IN_CAPS) &&
+      sink->previous_buffer_in_caps == FALSE) {
+    GST_DEBUG_OBJECT (sink,
+        "receiving new IN_CAPS buffers, clearing old streamheader");
+    g_slist_foreach (sink->streamheader, (GFunc) gst_data_unref, NULL);
+    g_slist_free (sink->streamheader);
+    sink->streamheader = NULL;
+  }
   /* if the incoming buffer is marked as IN CAPS, then we assume for now
    * it's a streamheader that needs to be sent to each new client, so we
    * put it on our internal list of streamheader buffers.
@@ -1465,6 +1479,7 @@ gst_multifdsink_chain (GstPad * pad, GstData * _data)
    * non IN_CAPS buffers so we properly keep track of clients that got
    * streamheaders. */
   if (GST_BUFFER_FLAG_IS_SET (buf, GST_BUFFER_IN_CAPS)) {
+    sink->previous_buffer_in_caps = TRUE;
     GST_DEBUG_OBJECT (sink,
         "appending IN_CAPS buffer with length %d to streamheader",
         GST_BUFFER_SIZE (buf));
@@ -1472,6 +1487,7 @@ gst_multifdsink_chain (GstPad * pad, GstData * _data)
     return;
   }
 
+  sink->previous_buffer_in_caps = FALSE;
   /* queue the buffer */
   gst_multifdsink_queue_buffer (sink, buf);
 
@@ -1675,6 +1691,9 @@ gst_multifdsink_close (GstMultiFdSink * this)
     gst_fdset_free (this->fdset);
     this->fdset = NULL;
   }
+
+  CLIENTS_LOCK_FREE (this);
+  g_hash_table_destroy (this->fd_hash);
 }
 
 static GstElementStateReturn
