@@ -30,6 +30,7 @@
 #include <gst/gstversion.h>
 #include <gst/gstinfo.h>
 #include <gst/gstutils.h>
+#include <gst/gstvalue.h>
 
 #include <string.h>
 #include <ctype.h>
@@ -190,6 +191,31 @@ id3_type_find (GstTypeFind * tf, gpointer unused)
   }
 }
 
+/*** application/x-ape **************************************************************/
+
+static GstStaticCaps apetag_caps = GST_STATIC_CAPS ("application/x-apetag");
+
+#define APETAG_CAPS gst_static_caps_get(&apetag_caps)
+static void
+apetag_type_find (GstTypeFind * tf, gpointer unused)
+{
+  guint8 *data;
+
+  /* APEv1/2 at start of file */
+  data = gst_type_find_peek (tf, 0, 8);
+  if (data && !memcmp (data, "APETAGEX", 8)) {
+    gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, APETAG_CAPS);
+    return;
+  }
+
+  /* APEv1/2 at end of file */
+  data = gst_type_find_peek (tf, -32, 8);
+  if (data && !memcmp (data, "APETAGEX", 8)) {
+    gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, APETAG_CAPS);
+    return;
+  }
+}
+
 /*** audio/x-ttafile *********************************************************/
 
 static GstStaticCaps tta_caps = GST_STATIC_CAPS ("audio/x-ttafile");
@@ -213,24 +239,72 @@ tta_type_find (GstTypeFind * tf, gpointer unused)
 static GstStaticCaps aac_caps = GST_STATIC_CAPS ("audio/mpeg, "
     "mpegversion = (int) { 2, 4 }, framed = (bool) false");
 #define AAC_CAPS (gst_static_caps_get(&aac_caps))
+#define AAC_AMOUNT (4096)
 static void
 aac_type_find (GstTypeFind * tf, gpointer unused)
 {
-  guint8 *data = gst_type_find_peek (tf, 0, 2);
+  guint8 *data = gst_type_find_peek (tf, 0, AAC_AMOUNT);
+  gint snc;
 
-  /* detect adts header
-   * note that this is a pretty lame typefind method (14 bits, 0.006%), so
-   * we'll only use LIKELY
+  /* detect adts header or adif header.
+   * The ADIF header is 4 bytes, that should be OK. The ADTS header, on
+   * the other hand, is 14 bits only, so we require one valid frame with
+   * again a valid syncpoint on the next one (28 bits) for certainty. We
+   * require 4 kB, which is quite a lot, since frames are generally 200-400
+   * bytes.
    */
-  if (data[0] == 0xFF && (data[1] & 0xF6) == 0xF0) {
-    gboolean mpegversion = (data[1] & 0x08) ? 2 : 4;
-    GstCaps *caps = gst_caps_new_simple ("audio/mpeg",
-        "framed", G_TYPE_BOOLEAN, FALSE,
-        "mpegversion", G_TYPE_INT, mpegversion,
-        NULL);
+  if (data) {
+    gint n;
 
-    gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, caps);
-    gst_caps_free (caps);
+    for (n = 0; n < AAC_AMOUNT - 3; n++) {
+      snc = GST_READ_UINT16_BE (&data[n]);
+      if ((snc & 0xfff6) == 0xfff0) {
+        /* ADTS header - find frame length */
+        gint len;
+
+        GST_DEBUG ("Found one ADTS syncpoint at offset 0x%x, tracing next...",
+            n);
+        if (AAC_AMOUNT - n < 5) {
+          GST_DEBUG ("Not enough data to parse ADTS header");
+          break;
+        }
+        len = ((data[n + 3] & 0x03) << 11) |
+            (data[n + 4] << 3) | ((data[n + 5] & 0xe0) >> 5);
+        if (n + len + 2 >= AAC_AMOUNT) {
+          GST_DEBUG ("Next frame is not within reach");
+          break;
+        } else if (len == 0) {
+          continue;
+        }
+
+        snc = GST_READ_UINT16_BE (&data[n + len]);
+        if ((snc & 0xfff6) == 0xfff0) {
+          gint mpegversion = (data[n + 1] & 0x08) ? 2 : 4;
+          GstCaps *caps = gst_caps_new_simple ("audio/mpeg",
+              "framed", G_TYPE_BOOLEAN, FALSE,
+              "mpegversion", G_TYPE_INT, mpegversion,
+              NULL);
+
+          gst_type_find_suggest (tf, GST_TYPE_FIND_LIKELY, caps);
+          gst_caps_free (caps);
+
+          GST_DEBUG ("Found ADTS-%d syncpoint at offset 0x%x (framelen %u)",
+              mpegversion, n, len);
+          break;
+        }
+
+        GST_DEBUG ("No next frame found... (should be at 0x%x)", n + len);
+      } else if (!memcmp (&data[n], "ADIF", 4)) {
+        /* ADIF header */
+        GstCaps *caps = gst_caps_new_simple ("audio/mpeg",
+            "framed", G_TYPE_BOOLEAN, FALSE,
+            "mpegversion", G_TYPE_INT, 4,
+            NULL);
+
+        gst_type_find_suggest (tf, GST_TYPE_FIND_LIKELY, caps);
+        gst_caps_free (caps);
+      }
+    }
   }
 }
 
@@ -274,7 +348,8 @@ static guint mp3types_freqs[3][3] = { {11025, 12000, 8000},
 
 static inline guint
 mp3_type_frame_length_from_header (guint32 header, guint * put_layer,
-    guint * put_channels, guint * put_bitrate, guint * put_samplerate)
+    guint * put_channels, guint * put_bitrate, guint * put_samplerate,
+    gboolean * may_be_free_format, gint possible_free_framelen)
 {
   guint bitrate, layer, length, mode, samplerate, version, channels;
 
@@ -301,7 +376,11 @@ mp3_type_frame_length_from_header (guint32 header, guint * put_layer,
 
   /* bitrate index */
   bitrate = header & 0xF;
-  if (bitrate == 15 || bitrate == 0)
+  if (bitrate == 0 && possible_free_framelen == -1) {
+    GST_LOG ("Possibly a free format mp3 - signalling");
+    *may_be_free_format = TRUE;
+  }
+  if (bitrate == 15 || (bitrate == 0 && possible_free_framelen == -1))
     return 0;
 
   /* ignore error correction, too */
@@ -320,15 +399,26 @@ mp3_type_frame_length_from_header (guint32 header, guint * put_layer,
 
   /* lookup */
   channels = (mode == 3) ? 1 : 2;
-  bitrate = mp3types_bitrates[version == 3 ? 0 : 1][layer - 1][bitrate];
   samplerate = mp3types_freqs[version > 0 ? version - 1 : 0][samplerate];
-
-  /* calculating */
-  if (layer == 1) {
-    length = ((12000 * bitrate / samplerate) + length) * 4;
+  if (bitrate == 0) {
+    if (layer == 1) {
+      length *= 4;
+      length += possible_free_framelen;
+      bitrate = length * samplerate / 48000;
+    } else {
+      length += possible_free_framelen;
+      bitrate = length * samplerate /
+          ((layer == 3 && version != 3) ? 72000 : 144000);
+    }
   } else {
-    length += ((layer == 3
-            && version != 3) ? 72000 : 144000) * bitrate / samplerate;
+    /* calculating */
+    bitrate = mp3types_bitrates[version == 3 ? 0 : 1][layer - 1][bitrate];
+    if (layer == 1) {
+      length = ((12000 * bitrate / samplerate) + length) * 4;
+    } else {
+      length += ((layer == 3
+              && version != 3) ? 72000 : 144000) * bitrate / samplerate;
+    }
   }
 
   GST_LOG ("mp3typefind: calculated mp3 frame length of %u bytes", length);
@@ -367,105 +457,210 @@ static GstStaticCaps mp3_caps = GST_STATIC_CAPS ("audio/mpeg, "
 static void
 mp3_type_find (GstTypeFind * tf, gpointer unused)
 {
+  guint64 length = gst_type_find_get_length (tf);
+  gint try;
   guint8 *data = NULL;
-  guint size = 0;
-  guint64 skipped = 0;
+  guint size;
+  guint64 skipped;
 
-  while (skipped < GST_MP3_TYPEFIND_TRY_SYNC) {
-    if (size <= 0) {
-      size = GST_MP3_TYPEFIND_SYNC_SIZE * 2;
-      do {
-        size /= 2;
-        data = gst_type_find_peek (tf, skipped, size);
-      } while (size > 10 && !data);
-      if (!data)
-        break;
-    }
-    if (*data == 0xFF) {
-      guint8 *head_data = NULL;
-      guint layer, bitrate, samplerate, channels;
-      guint found = 0;          /* number of valid headers found */
-      guint64 offset = skipped;
+  for (try = 0; try < 2; try++) {
+    gint last_free_offset = -1, last_free_framelen = -1;
+    guint64 start_off = (try == 0) ? 0 : length / 2;
 
-      while (found < GST_MP3_TYPEFIND_TRY_HEADERS) {
-        guint32 head;
-        guint length;
-        guint prev_layer = 0, prev_bitrate = 0,
-            prev_channels = 0, prev_samplerate = 0;
+    if (try != 0 && start_off == 0)
+      return;
 
-        if (offset + 4 <= skipped + size) {
-          head_data = data + offset - skipped;
-        } else {
-          head_data = gst_type_find_peek (tf, offset, 4);
-        }
-        if (!head_data)
+    size = 0;
+    skipped = 0;
+    while (skipped < GST_MP3_TYPEFIND_TRY_SYNC) {
+      if (size <= 0) {
+        size = GST_MP3_TYPEFIND_SYNC_SIZE * 2;
+        do {
+          size /= 2;
+          data = gst_type_find_peek (tf, skipped + start_off, size);
+        } while (size > 10 && !data);
+        if (!data)
           break;
-        head = GST_READ_UINT32_BE (head_data);
-        if (!(length = mp3_type_frame_length_from_header (head, &layer,
-                    &channels, &bitrate, &samplerate))) {
-          GST_LOG ("%d. header at offset %" G_GUINT64_FORMAT
-              " (0x%X) was not an mp3 header", found + 1, offset,
-              (guint) offset);
-          break;
-        }
-        if ((prev_layer && prev_layer != layer) ||
-            /* (prev_bitrate && prev_bitrate != bitrate) || <-- VBR */
-            (prev_samplerate && prev_samplerate != samplerate) ||
-            (prev_channels && prev_channels != channels)) {
-          /* this means an invalid property, or a change, which might mean
-           * that this is not a mp3 but just a random bytestream. It could
-           * be a freaking funky encoded mp3 though. We'll just not count
-           * this header*/
-          prev_layer = layer;
-          prev_bitrate = bitrate;
-          prev_channels = channels;
-          prev_samplerate = samplerate;
-        } else {
-          found++;
-          GST_LOG ("found %d. header at offset %" G_GUINT64_FORMAT " (0x%X)",
-              found, offset, (guint) offset);
-        }
-        offset += length;
       }
-      g_assert (found <= GST_MP3_TYPEFIND_TRY_HEADERS);
-      if (found == GST_MP3_TYPEFIND_TRY_HEADERS ||
-          (found >= GST_MP3_TYPEFIND_MIN_HEADERS && head_data == NULL)) {
-        /* we can make a valid guess */
-        guint probability = found * GST_TYPE_FIND_MAXIMUM *
-            (GST_MP3_TYPEFIND_TRY_SYNC - skipped) /
-            GST_MP3_TYPEFIND_TRY_HEADERS / GST_MP3_TYPEFIND_TRY_SYNC;
+      if (*data == 0xFF) {
+        guint8 *head_data = NULL;
+        guint layer = 0, bitrate, samplerate, channels;
+        guint found = 0;        /* number of valid headers found */
+        guint64 offset = skipped;
 
-        if (probability < GST_TYPE_FIND_MINIMUM)
-          probability = GST_TYPE_FIND_MINIMUM;
-        GST_INFO
-            ("audio/mpeg calculated %u  =  %u  *  %u / %u  *  (%u - %u) / %u",
-            probability, GST_TYPE_FIND_MAXIMUM, found,
-            GST_MP3_TYPEFIND_TRY_HEADERS, GST_MP3_TYPEFIND_TRY_SYNC, skipped,
-            GST_MP3_TYPEFIND_TRY_SYNC);
-        /* make sure we're not id3 tagged */
-        head_data = gst_type_find_peek (tf, -128, 3);
-        if (!head_data) {
-          probability = probability * 4 / 5;
-        } else if (memcmp (head_data, "TAG", 3) == 0) {
-          probability = 0;
-        }
-        g_assert (probability <= GST_TYPE_FIND_MAXIMUM);
-        if (probability > 0) {
-          GstCaps *caps;
+        while (found < GST_MP3_TYPEFIND_TRY_HEADERS) {
+          guint32 head;
+          guint length;
+          guint prev_layer = 0, prev_bitrate = 0,
+              prev_channels = 0, prev_samplerate = 0;
+          gboolean free = FALSE;
 
-          g_assert (layer > 0);
-          caps = gst_caps_copy (MP3_CAPS);
-          gst_structure_set (gst_caps_get_structure (caps, 0), "layer",
-              G_TYPE_INT, layer, 0);
-          gst_type_find_suggest (tf, probability, caps);
-          gst_caps_free (caps);
+          if (offset + 4 <= skipped + size) {
+            head_data = data + offset - skipped;
+          } else {
+            head_data = gst_type_find_peek (tf, offset + start_off, 4);
+          }
+          if (!head_data)
+            break;
+          head = GST_READ_UINT32_BE (head_data);
+          if (!(length = mp3_type_frame_length_from_header (head, &layer,
+                      &channels, &bitrate, &samplerate, &free,
+                      last_free_framelen))) {
+            if (free) {
+              if (last_free_offset == -1)
+                last_free_offset = offset;
+              else {
+                last_free_framelen = offset - last_free_offset;
+                offset = last_free_offset;
+                continue;
+              }
+            } else {
+              last_free_framelen = -1;
+            }
+            GST_LOG ("%d. header at offset %" G_GUINT64_FORMAT
+                " (0x%X) was not an mp3 header (possibly-free: %s)",
+                found + 1, offset, (guint) offset, free ? "yes" : "no");
+            break;
+          }
+          if ((prev_layer && prev_layer != layer) ||
+              /* (prev_bitrate && prev_bitrate != bitrate) || <-- VBR */
+              (prev_samplerate && prev_samplerate != samplerate) ||
+              (prev_channels && prev_channels != channels)) {
+            /* this means an invalid property, or a change, which might mean
+             * that this is not a mp3 but just a random bytestream. It could
+             * be a freaking funky encoded mp3 though. We'll just not count
+             * this header*/
+            prev_layer = layer;
+            prev_bitrate = bitrate;
+            prev_channels = channels;
+            prev_samplerate = samplerate;
+          } else {
+            found++;
+            GST_LOG ("found %d. header at offset %" G_GUINT64_FORMAT " (0x%X)",
+                found, offset, (guint) offset);
+          }
+          offset += length;
         }
+        g_assert (found <= GST_MP3_TYPEFIND_TRY_HEADERS);
+        if (found == GST_MP3_TYPEFIND_TRY_HEADERS ||
+            (found >= GST_MP3_TYPEFIND_MIN_HEADERS && head_data == NULL)) {
+          /* we can make a valid guess */
+          guint probability = found * GST_TYPE_FIND_MAXIMUM *
+              (GST_MP3_TYPEFIND_TRY_SYNC - skipped) /
+              GST_MP3_TYPEFIND_TRY_HEADERS / GST_MP3_TYPEFIND_TRY_SYNC;
+
+          if (probability < GST_TYPE_FIND_MINIMUM)
+            probability = GST_TYPE_FIND_MINIMUM;
+          probability /= (try + 1);
+          GST_INFO
+              ("audio/mpeg calculated %u  =  %u  *  %u / %u  *  (%u - %u) / %u",
+              probability, GST_TYPE_FIND_MAXIMUM, found,
+              GST_MP3_TYPEFIND_TRY_HEADERS, GST_MP3_TYPEFIND_TRY_SYNC, skipped,
+              GST_MP3_TYPEFIND_TRY_SYNC);
+          /* make sure we're not id3 tagged */
+          head_data = gst_type_find_peek (tf, -128, 3);
+          if (!head_data) {
+            probability = probability * 4 / 5;
+          } else if (memcmp (head_data, "TAG", 3) == 0) {
+            probability = 0;
+          }
+          g_assert (probability <= GST_TYPE_FIND_MAXIMUM);
+          if (probability > 0) {
+            GstCaps *caps;
+
+            g_assert (layer > 0);
+            caps = gst_caps_copy (MP3_CAPS);
+            gst_structure_set (gst_caps_get_structure (caps, 0), "layer",
+                G_TYPE_INT, layer, 0);
+            gst_type_find_suggest (tf, probability, caps);
+            gst_caps_free (caps);
+          }
+          return;
+        }
+      }
+      data++;
+      skipped++;
+      size--;
+    }
+  }
+}
+
+/*** audio/x-ac3 **************************************************************/
+static GstStaticCaps ac3_caps = GST_STATIC_CAPS ("audio/x-ac3");
+
+#define AC3_CAPS (gst_static_caps_get(&ac3_caps))
+
+static void
+ac3_type_find (GstTypeFind * tf, gpointer unused)
+{
+  guint8 *data = gst_type_find_peek (tf, 0, 2);
+
+  if (data) {
+    /* pretty lame method... */
+    if (data[0] == 0x0b && data[1] == 0x77) {
+      gst_type_find_suggest (tf, GST_TYPE_FIND_POSSIBLE, AC3_CAPS);
+      return;
+    }
+  }
+}
+
+/*** wavpack *****************************************************************/
+static GstStaticCaps wavpack_caps =
+GST_STATIC_CAPS ("audio/x-wavpack, framed = (boolean) false");
+
+#define WAVPACK_CAPS (gst_static_caps_get(&wavpack_caps))
+
+static GstStaticCaps wavpack_correction_caps =
+GST_STATIC_CAPS ("audio/x-wavpack-correction, framed = (boolean) false");
+
+#define WAVPACK_CORRECTION_CAPS (gst_static_caps_get(&wavpack_correction_caps))
+
+static void
+wavpack_type_find (GstTypeFind * tf, gpointer unused)
+{
+  guint8 *data = gst_type_find_peek (tf, 0, 32);
+
+  if (!data)
+    return;
+
+  if (data[0] == 'w' && data[1] == 'v' && data[2] == 'p' && data[3] == 'k') {
+    guint32 blocksize, sublen;
+    gint32 left;
+
+    blocksize = GST_READ_UINT32_LE (data + 4);
+    /* peek from offset 0, otherwise it won't work with apedemux */
+    data = gst_type_find_peek (tf, 0, 32 + blocksize);
+    if (!data)
+      return;
+    data += 32;
+    left = (gint32) blocksize;
+    while (left > 2) {
+      sublen = ((guint32) data[1]) << 1;
+      if (data[0] & 0x80) {
+        sublen |= (((guint32) data[2]) << 9) | (((guint32) data[3]) << 17);
+        sublen += 1 + 3;        /* id + length */
+      } else {
+        sublen += 1 + 1;        /* id + length */
+      }
+      if (sublen > blocksize)
         return;
+      if ((data[0] & 0x20) == 0) {
+        switch (data[0] & 0x0f) {
+          case 0xa:            /* ID_WV_BITSTREAM  */
+          case 0xc:            /* ID_WVX_BITSTREAM */
+            gst_type_find_suggest (tf, GST_TYPE_FIND_LIKELY, WAVPACK_CAPS);
+            return;
+          case 0xb:            /* ID_WVC_BITSTREAM */
+            gst_type_find_suggest (tf, GST_TYPE_FIND_LIKELY,
+                WAVPACK_CORRECTION_CAPS);
+            return;
+          default:
+            break;
+        }
       }
+      left -= sublen;
+      data += sublen;
     }
-    data++;
-    skipped++;
-    size--;
   }
 }
 
@@ -486,6 +681,13 @@ static GstStaticCaps mpeg_sys_caps = GST_STATIC_CAPS ("video/mpeg, "
 					 (((guint8 *)data)[1] == 0x00) &&	\
 					 (((guint8 *)data)[2] == 0x01) &&	\
 					 ((((guint8 *)data)[3] & 0x80) == 0x80))
+#define IS_MPEG_PES_HEADER(data)        ((((guint8 *)data)[0] == 0x00) &&	\
+					 (((guint8 *)data)[1] == 0x00) &&	\
+					 (((guint8 *)data)[2] == 0x01) && \
+					 ((((guint8 *)data)[3] == 0xE0) || \
+					  (((guint8 *)data)[3] == 0xC0) || \
+					  (((guint8 *)data)[3] == 0xBD)))
+
 static void
 mpeg2_sys_type_find (GstTypeFind * tf, gpointer unused)
 {
@@ -507,7 +709,15 @@ mpeg2_sys_type_find (GstTypeFind * tf, gpointer unused)
       gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, caps);
       gst_caps_free (caps);
     }
+  } else if (data && IS_MPEG_PES_HEADER (data)) {
+    /* PES stream */
+    GstCaps *caps = gst_caps_copy (MPEG_SYS_CAPS);
+
+    gst_structure_set (gst_caps_get_structure (caps, 0), "mpegversion",
+        G_TYPE_INT, 2, 0);
+    gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, caps);
   }
+
 };
 
 /* ATTANTION: ugly return value:
@@ -770,15 +980,17 @@ qt_type_find (GstTypeFind * tf, gpointer unused)
   guint8 *data;
   guint tip = 0;
   guint64 offset = 0;
+  guint64 size;
 
   while ((data = gst_type_find_peek (tf, offset, 8)) != NULL) {
-    if (strncmp (&data[4], "wide", 4) != 0 &&
-        strncmp (&data[4], "moov", 4) != 0 &&
-        strncmp (&data[4], "mdat", 4) != 0 &&
-        strncmp (&data[4], "pnot", 4) != 0 &&
-        strncmp (&data[4], "PICT", 4) != 0 &&
-        strncmp (&data[4], "ftyp", 4) != 0 &&
-        strncmp (&data[4], "free", 4) != 0) {
+    if (strncmp ((const char *) &data[4], "wide", 4) != 0 &&
+        strncmp ((const char *) &data[4], "moov", 4) != 0 &&
+        strncmp ((const char *) &data[4], "mdat", 4) != 0 &&
+        strncmp ((const char *) &data[4], "pnot", 4) != 0 &&
+        strncmp ((const char *) &data[4], "PICT", 4) != 0 &&
+        strncmp ((const char *) &data[4], "ftyp", 4) != 0 &&
+        strncmp ((const char *) &data[4], "free", 4) != 0 &&
+        strncmp ((const char *) &data[4], "skip", 4) != 0) {
       tip = 0;
       break;
     }
@@ -788,9 +1000,20 @@ qt_type_find (GstTypeFind * tf, gpointer unused)
       tip = GST_TYPE_FIND_MAXIMUM;
       break;
     }
-    offset += GST_READ_UINT32_BE (data);
-    if (GST_READ_UINT32_BE (data) < 8)
-      break;
+    size = GST_READ_UINT32_BE (data);
+    if (size == 1) {
+      guint8 *sizedata;
+
+      sizedata = gst_type_find_peek (tf, offset + 8, 8);
+      if (sizedata == NULL)
+        break;
+
+      size = GST_READ_UINT64_BE (sizedata);
+    } else {
+      if (size < 8)
+        break;
+    }
+    offset += size;
   }
   if (tip > 0) {
     gst_type_find_suggest (tf, tip, QT_CAPS);
@@ -809,11 +1032,28 @@ aiff_type_find (GstTypeFind * tf, gpointer unused)
 
   if (data && memcmp (data, "FORM", 4) == 0) {
     data += 8;
-    if (memcmp (data, "AIFF", 4) == 0)
+    if (memcmp (data, "AIFF", 4) == 0 || memcmp (data, "AIFC", 4) == 0)
       gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, AIFF_CAPS);
   }
 }
 
+/*** audio/x-aiff *********************************************/
+
+static GstStaticCaps svx_caps = GST_STATIC_CAPS ("audio/x-svx");
+
+#define SVX_CAPS gst_static_caps_get(&svx_caps)
+static void
+svx_type_find (GstTypeFind * tf, gpointer unused)
+{
+  guint8 *data = gst_type_find_peek (tf, 0, 4);
+
+  if (data && memcmp (data, "FORM", 4) == 0) {
+    data += 8;
+    if (memcmp (data, "8SVX", 4) == 0 || memcmp (data, "16SV", 4) == 0)
+      gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, SVX_CAPS);
+  }
+}
+
 /*** audio/x-shorten ****************************************/
 
 static GstStaticCaps shn_caps = GST_STATIC_CAPS ("audio/x-shorten");
@@ -859,7 +1099,26 @@ m4a_type_find (GstTypeFind * tf, gpointer unused)
   guint8 *data = gst_type_find_peek (tf, 4, 8);
 
   if (data && memcmp (data, "ftypM4A ", 8) == 0) {
-    gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, AAC_CAPS);
+    gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, M4A_CAPS);
+  }
+}
+
+/*** application/x-3gp *********************************************/
+
+/*
+ * The Q is there because variables can't start with a number.
+ */
+
+static GstStaticCaps q3gp_caps = GST_STATIC_CAPS ("application/x-3gp");
+
+#define Q3GP_CAPS (gst_static_caps_get(&q3gp_caps))
+static void
+q3gp_type_find (GstTypeFind * tf, gpointer unused)
+{
+  guint8 *data = gst_type_find_peek (tf, 4, 8);
+
+  if (data && memcmp (data, "ftyp3gp4", 8) == 0) {
+    gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, Q3GP_CAPS);
   }
 }
 
@@ -867,6 +1126,7 @@ m4a_type_find (GstTypeFind * tf, gpointer unused)
 
 static GstStaticCaps mod_caps = GST_STATIC_CAPS ("audio/x-mod");
 
+
 #define MOD_CAPS gst_static_caps_get(&mod_caps)
 /* FIXME: M15 CheckType to do */
 static void
@@ -1066,6 +1326,61 @@ tiff_type_find (GstTypeFind * tf, gpointer ununsed)
   }
 }
 
+static GstStaticCaps sds_caps = GST_STATIC_CAPS ("audio/x-sds");
+
+#define SDS_CAPS (gst_static_caps_get(&sds_caps))
+static void
+sds_type_find (GstTypeFind * tf, gpointer ununsed)
+{
+  guint8 *data = gst_type_find_peek (tf, 0, 4);
+  guint8 mask[4] = { 0xFF, 0xFF, 0x80, 0xFF };
+  guint8 match[4] = { 0xF0, 0x7E, 0, 0x01 };
+  gint x;
+
+  if (data) {
+    for (x = 0; x < 4; x++) {
+      if ((data[x] & mask[x]) != match[x]) {
+        return;
+      }
+    }
+    gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, SDS_CAPS);
+  }
+}
+
+static GstStaticCaps ircam_caps = GST_STATIC_CAPS ("audio/x-ircam");
+
+#define IRCAM_CAPS (gst_static_caps_get(&ircam_caps))
+static void
+ircam_type_find (GstTypeFind * tf, gpointer ununsed)
+{
+  guint8 *data = gst_type_find_peek (tf, 0, 4);
+  guint8 mask[4] = { 0xFF, 0xFF, 0xF8, 0xFF };
+  guint8 match[4] = { 0x64, 0xA3, 0x00, 0x00 };
+  gint x;
+  gboolean matched = TRUE;
+
+  if (!data) {
+    return;
+  }
+  for (x = 0; x < 4; x++) {
+    if ((data[x] & mask[x]) != match[x]) {
+      matched = FALSE;
+    }
+  }
+  if (matched) {
+    gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, IRCAM_CAPS);
+    return;
+  }
+  // now try the reverse version
+  matched = TRUE;
+  for (x = 0; x < 4; x++) {
+    if ((data[x] & mask[3 - x]) != match[3 - x]) {
+      matched = FALSE;
+    }
+  }
+}
+
+
 /*** video/x-matroska ********************/
 static GstStaticCaps matroska_caps = GST_STATIC_CAPS ("video/x-matroska");
 
@@ -1145,8 +1460,36 @@ dv_type_find (GstTypeFind * tf, gpointer private)
   }
 }
 
-/*** audio/x-vorbis ***********************************************************/
 
+/*** application/ogg and application/x-annodex *****************************/
+static GstStaticCaps ogg_caps = GST_STATIC_CAPS ("application/ogg");
+static GstStaticCaps annodex_caps = GST_STATIC_CAPS ("application/x-annodex");
+static GstStaticCaps ogganx_caps =
+    GST_STATIC_CAPS ("application/ogg; application/x-annodex");
+#define OGGANX_CAPS (gst_static_caps_get(&ogganx_caps))
+
+static void
+ogganx_type_find (GstTypeFind * tf, gpointer private)
+{
+  guint8 *data = gst_type_find_peek (tf, 28, 8);
+  gboolean is_annodex = FALSE;
+
+  if ((data != NULL) && (memcmp (data, "fishead\0", 8) == 0)) {
+    is_annodex = TRUE;
+  }
+
+  data = gst_type_find_peek (tf, 0, 4);
+  if ((data != NULL) && (memcmp (data, "OggS", 4) == 0)) {
+    if (is_annodex == TRUE) {
+      gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM,
+          gst_static_caps_get (&annodex_caps));
+    }
+    gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM,
+        gst_static_caps_get (&ogg_caps));
+  }
+}
+
+/*** audio/x-vorbis ***********************************************************/
 static GstStaticCaps vorbis_caps = GST_STATIC_CAPS ("audio/x-vorbis");
 
 #define VORBIS_CAPS (gst_static_caps_get(&vorbis_caps))
@@ -1243,6 +1586,21 @@ ogmaudio_type_find (GstTypeFind * tf, gpointer private)
   }
 }
 
+static GstStaticCaps ogmtext_caps = GST_STATIC_CAPS ("application/x-ogm-text");
+
+#define OGMTEXT_CAPS (gst_static_caps_get(&ogmtext_caps))
+static void
+ogmtext_type_find (GstTypeFind * tf, gpointer private)
+{
+  guint8 *data = gst_type_find_peek (tf, 0, 9);
+
+  if (data) {
+    if (memcmp (data, "\001text\000\000\000\000", 9) != 0)
+      return;
+    gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, OGMTEXT_CAPS);
+  }
+}
+
 /*** audio/x-speex ***********************************************************/
 
 static GstStaticCaps speex_caps = GST_STATIC_CAPS ("audio/x-speex");
@@ -1279,11 +1637,78 @@ speex_type_find (GstTypeFind * tf, gpointer private)
   }
 }
 
-/*** generic typefind for streams that have some data at a specific position***/
+/*** application/x-ogg-skeleton ***********************************************************/
+static GstStaticCaps ogg_skeleton_caps =
+GST_STATIC_CAPS ("application/x-ogg-skeleton");
+#define OGG_SKELETON_CAPS (gst_static_caps_get(&ogg_skeleton_caps))
+static void
+oggskel_type_find (GstTypeFind * tf, gpointer private)
+{
+  guint8 *data = gst_type_find_peek (tf, 0, 12);
 
+  if (data) {
+    /* 8 byte string "fishead\0" for the ogg skeleton stream */
+    if (memcmp (data, "fishead\0", 8) != 0)
+      return;
+    data += 8;
+
+    /* Require that the header contains version 3.0 */
+    if (GST_READ_UINT16_LE (data) != 3)
+      return;
+    data += 2;
+    if (GST_READ_UINT16_LE (data) != 0)
+      return;
+
+    gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, OGG_SKELETON_CAPS);
+  }
+}
+
+static GstStaticCaps cmml_caps = GST_STATIC_CAPS ("text/x-cmml");
+
+#define CMML_CAPS (gst_static_caps_get(&cmml_caps))
+static void
+cmml_type_find (GstTypeFind * tf, gpointer private)
+{
+  guint8 *data = gst_type_find_peek (tf, 0, 12);
+
+  if (data) {
+    /* 8 byte string "CMML\0\0\0\0" for the magic number */
+    if (memcmp (data, "CMML\0\0\0\0", 8) != 0)
+      return;
+    data += 8;
+
+    /* Require that the header contains at least version 2.0 */
+    if (GST_READ_UINT16_LE (data) < 2)
+      return;
+    data += 2;
+
+    if (GST_READ_UINT16_LE (data) != 0)
+      return;
+
+    gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, CMML_CAPS);
+  }
+}
+
+static GstStaticCaps au_caps = GST_STATIC_CAPS ("audio/x-au");
+
+#define AU_CAPS (gst_static_caps_get (&au_caps))
+static void
+au_type_find (GstTypeFind * tf, gpointer private)
+{
+  guint8 *data = gst_type_find_peek (tf, 0, 4);
+
+  if (data) {
+    if (!(GST_READ_UINT32_LE (data) == GST_MAKE_FOURCC ('.', 's', 'n', 'd') ||
+            GST_READ_UINT32_BE (data) == GST_MAKE_FOURCC ('.', 's', 'n', 'd')))
+      return;
+    gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, AU_CAPS);
+  }
+}
+
+/*** generic typefind for streams that have some data at a specific position***/
 typedef struct
 {
-  guint8 *data;
+  const guint8 *data;
   guint size;
   guint probability;
   GstCaps *caps;
@@ -1307,7 +1732,7 @@ start_with_type_find (GstTypeFind * tf, gpointer private)
 #define TYPE_FIND_REGISTER_START_WITH(plugin,name,rank,ext,_data,_size,_probability)\
 G_BEGIN_DECLS{									\
   GstTypeFindData *sw_data = g_new (GstTypeFindData, 1);			\
-  sw_data->data = _data;						      	\
+  sw_data->data = (const guint8*) _data;				      	\
   sw_data->size = _size;						      	\
   sw_data->probability = _probability;						\
   sw_data->caps = gst_caps_new_simple (name, NULL);			\
@@ -1333,7 +1758,7 @@ riff_type_find (GstTypeFind * tf, gpointer private)
 #define TYPE_FIND_REGISTER_RIFF(plugin,name,rank,ext,_data)			\
 G_BEGIN_DECLS{									\
   GstTypeFindData *sw_data = g_new (GstTypeFindData, 1);			\
-  sw_data->data = _data;						      	\
+  sw_data->data = (const guint8*) _data;				      	\
   sw_data->size = 4;							      	\
   sw_data->probability = GST_TYPE_FIND_MAXIMUM;					\
   sw_data->caps = gst_caps_new_simple (name, NULL);			\
@@ -1361,21 +1786,31 @@ plugin_init (GstPlugin * plugin)
   static gchar *flx_exts[] = { "flc", "fli", NULL };
   static gchar *id3_exts[] =
       { "mp3", "mp2", "mp1", "mpga", "ogg", "flac", "tta", NULL };
+  static gchar *apetag_exts[] = { "ape", "mpc", "wv", NULL };
   static gchar *tta_exts[] = { "tta", NULL };
   static gchar *mod_exts[] = { "669", "amf", "dsm", "gdm", "far", "imf",
     "it", "med", "mod", "mtm", "okt", "sam",
     "s3m", "stm", "stx", "ult", "xm", NULL
   };
   static gchar *mp3_exts[] = { "mp3", "mp2", "mp1", "mpga", NULL };
+  static gchar *ac3_exts[] = { "ac3", NULL };
+  static gchar *musepack_exts[] = { "mpc", NULL };
   static gchar *mpeg_sys_exts[] = { "mpe", "mpeg", "mpg", NULL };
   static gchar *mpeg_video_exts[] = { "mpv", "mpeg", "mpg", NULL };
-  static gchar *ogg_exts[] = { "ogg", "ogm", NULL };
+  static gchar *ogg_exts[] = { "anx", "ogg", "ogm", NULL };
   static gchar *qt_exts[] = { "mov", NULL };
   static gchar *rm_exts[] = { "ra", "ram", "rm", "rmvb", NULL };
   static gchar *swf_exts[] = { "swf", "swfl", NULL };
   static gchar *utf8_exts[] = { "txt", NULL };
   static gchar *wav_exts[] = { "wav", NULL };
   static gchar *aiff_exts[] = { "aiff", "aif", "aifc", NULL };
+  static gchar *svx_exts[] = { "iff", "svx", NULL };
+  static gchar *paris_exts[] = { "paf", NULL };
+  static gchar *nist_exts[] = { "nist", NULL };
+  static gchar *voc_exts[] = { "voc", NULL };
+  static gchar *sds_exts[] = { "sds", NULL };
+  static gchar *ircam_exts[] = { "sf", NULL };
+  static gchar *w64_exts[] = { "w64", NULL };
   static gchar *shn_exts[] = { "shn", NULL };
   static gchar *ape_exts[] = { "ape", NULL };
   static gchar *uri_exts[] = { "ram", NULL };
@@ -1386,6 +1821,7 @@ plugin_init (GstPlugin * plugin)
   static gchar *tiff_exts[] = { "tif", "tiff", NULL };
   static gchar *matroska_exts[] = { "mkv", "mka", NULL };
   static gchar *dv_exts[] = { "dv", "dif", NULL };
+  static gchar *amr_exts[] = { "amr", NULL };
   static gchar *sid_exts[] = { "sid", NULL };
   static gchar *xcf_exts[] = { "xcf", NULL };
   static gchar *mng_exts[] = { "mng", NULL };
@@ -1397,7 +1833,11 @@ plugin_init (GstPlugin * plugin)
   static gchar *zip_exts[] = { "zip", NULL };
   static gchar *compress_exts[] = { "Z", NULL };
   static gchar *m4a_exts[] = { "m4a", NULL };
+  static gchar *q3gp_exts[] = { "3gp", NULL };
   static gchar *aac_exts[] = { "aac", NULL };
+  static gchar *spc_exts[] = { "spc", NULL };
+  static gchar *wavpack_exts[] = { "wv", "wvp", NULL };
+  static gchar *wavpack_correction_exts[] = { "wvc", NULL };
 
   GST_DEBUG_CATEGORY_INIT (type_find_debug, "typefindfunctions",
       GST_DEBUG_FG_GREEN | GST_DEBUG_BG_RED, "generic type find functions");
@@ -1407,30 +1847,40 @@ plugin_init (GstPlugin * plugin)
       asf_exts,
       "\060\046\262\165\216\146\317\021\246\331\000\252\000\142\316\154", 16,
       GST_TYPE_FIND_MAXIMUM);
-  TYPE_FIND_REGISTER_START_WITH (plugin, "audio/x-au", GST_RANK_MARGINAL,
-      au_exts, ".snd", 4, GST_TYPE_FIND_MAXIMUM);
+  /* -1 so id3v1 or apev1/2 are detected with higher preference */
+  TYPE_FIND_REGISTER_START_WITH (plugin, "audio/x-musepack", GST_RANK_PRIMARY,
+      musepack_exts, "MP+", 3, GST_TYPE_FIND_MAXIMUM - 1);
+  TYPE_FIND_REGISTER (plugin, "audio/x-au", GST_RANK_MARGINAL, au_type_find,
+      au_exts, AU_CAPS, NULL);
   TYPE_FIND_REGISTER_RIFF (plugin, "video/x-msvideo", GST_RANK_PRIMARY,
       avi_exts, "AVI ");
   TYPE_FIND_REGISTER_RIFF (plugin, "video/x-cdxa", GST_RANK_PRIMARY,
       cdxa_exts, "CDXA");
+  TYPE_FIND_REGISTER_START_WITH (plugin, "video/x-vcd", GST_RANK_PRIMARY,
+      cdxa_exts, "\000\377\377\377\377\377\377\377\377\377\377\000", 12,
+      GST_TYPE_FIND_MAXIMUM);
   TYPE_FIND_REGISTER_START_WITH (plugin, "audio/x-flac", GST_RANK_PRIMARY,
       flac_exts, "fLaC", 4, GST_TYPE_FIND_MAXIMUM);
   TYPE_FIND_REGISTER (plugin, "video/x-fli", GST_RANK_MARGINAL, flx_type_find,
       flx_exts, FLX_CAPS, NULL);
   TYPE_FIND_REGISTER (plugin, "application/x-id3", GST_RANK_PRIMARY,
       id3_type_find, id3_exts, ID3_CAPS, NULL);
+  TYPE_FIND_REGISTER (plugin, "application/x-apetag", GST_RANK_PRIMARY,
+      apetag_type_find, apetag_exts, APETAG_CAPS, NULL);
   TYPE_FIND_REGISTER (plugin, "audio/x-ttafile", GST_RANK_PRIMARY,
       tta_type_find, tta_exts, TTA_CAPS, NULL);
   TYPE_FIND_REGISTER (plugin, "audio/x-mod", GST_RANK_SECONDARY, mod_type_find,
       mod_exts, MOD_CAPS, NULL);
   TYPE_FIND_REGISTER (plugin, "audio/mpeg", GST_RANK_PRIMARY, mp3_type_find,
       mp3_exts, MP3_CAPS, NULL);
+  TYPE_FIND_REGISTER (plugin, "audio/x-ac3", GST_RANK_PRIMARY, ac3_type_find,
+      ac3_exts, AC3_CAPS, NULL);
   TYPE_FIND_REGISTER (plugin, "video/mpeg1", GST_RANK_PRIMARY,
       mpeg1_sys_type_find, mpeg_sys_exts, MPEG_SYS_CAPS, NULL);
   TYPE_FIND_REGISTER (plugin, "video/mpeg2", GST_RANK_SECONDARY,
       mpeg2_sys_type_find, mpeg_sys_exts, MPEG_SYS_CAPS, NULL);
-  TYPE_FIND_REGISTER_START_WITH (plugin, "application/ogg", GST_RANK_PRIMARY,
-      ogg_exts, "OggS", 4, GST_TYPE_FIND_MAXIMUM);
+  TYPE_FIND_REGISTER (plugin, "application/ogg", GST_RANK_PRIMARY,
+      ogganx_type_find, ogg_exts, OGGANX_CAPS, NULL);
   TYPE_FIND_REGISTER (plugin, "video/mpeg", GST_RANK_SECONDARY,
       mpeg_video_type_find, mpeg_video_exts, MPEG_VIDEO_CAPS, NULL);
   TYPE_FIND_REGISTER (plugin, "video/mpeg-stream", GST_RANK_MARGINAL,
@@ -1449,6 +1899,22 @@ plugin_init (GstPlugin * plugin)
       "WAVE");
   TYPE_FIND_REGISTER (plugin, "audio/x-aiff", GST_RANK_SECONDARY,
       aiff_type_find, aiff_exts, AIFF_CAPS, NULL);
+  TYPE_FIND_REGISTER (plugin, "audio/x-svx", GST_RANK_SECONDARY,
+      svx_type_find, svx_exts, SVX_CAPS, NULL);
+  TYPE_FIND_REGISTER_START_WITH (plugin, "audio/x-paris",
+      GST_RANK_SECONDARY, paris_exts, " paf", 4, GST_TYPE_FIND_MAXIMUM);
+  TYPE_FIND_REGISTER_START_WITH (plugin, "audio/x-paris",
+      GST_RANK_SECONDARY, paris_exts, "fap ", 4, GST_TYPE_FIND_MAXIMUM);
+  TYPE_FIND_REGISTER_START_WITH (plugin, "audio/x-nist",
+      GST_RANK_SECONDARY, nist_exts, "NIST", 4, GST_TYPE_FIND_MAXIMUM);
+  TYPE_FIND_REGISTER_START_WITH (plugin, "audio/x-voc",
+      GST_RANK_SECONDARY, voc_exts, "Creative", 8, GST_TYPE_FIND_MAXIMUM);
+  TYPE_FIND_REGISTER (plugin, "audio/x-sds", GST_RANK_SECONDARY,
+      sds_type_find, sds_exts, SDS_CAPS, NULL);
+  TYPE_FIND_REGISTER (plugin, "audio/x-ircam", GST_RANK_SECONDARY,
+      ircam_type_find, ircam_exts, IRCAM_CAPS, NULL);
+  TYPE_FIND_REGISTER_START_WITH (plugin, "audio/x-w64",
+      GST_RANK_SECONDARY, w64_exts, "riff", 4, GST_TYPE_FIND_MAXIMUM);
   TYPE_FIND_REGISTER (plugin, "audio/x-shorten", GST_RANK_SECONDARY,
       shn_type_find, shn_exts, SHN_CAPS, NULL);
   TYPE_FIND_REGISTER (plugin, "application/x-ape", GST_RANK_SECONDARY,
@@ -1467,6 +1933,10 @@ plugin_init (GstPlugin * plugin)
       matroska_type_find, matroska_exts, MATROSKA_CAPS, NULL);
   TYPE_FIND_REGISTER (plugin, "video/x-dv", GST_RANK_SECONDARY, dv_type_find,
       dv_exts, DV_CAPS, NULL);
+  TYPE_FIND_REGISTER_START_WITH (plugin, "audio/x-amr-nb-sh", GST_RANK_PRIMARY,
+      amr_exts, "#!AMR", 5, GST_TYPE_FIND_LIKELY);
+  TYPE_FIND_REGISTER_START_WITH (plugin, "audio/x-amr-wb-sh", GST_RANK_PRIMARY,
+      amr_exts, "#!AMR-WB", 7, GST_TYPE_FIND_MAXIMUM);
   TYPE_FIND_REGISTER_START_WITH (plugin, "audio/x-sid", GST_RANK_MARGINAL,
       sid_exts, "PSID", 4, GST_TYPE_FIND_MAXIMUM);
   TYPE_FIND_REGISTER_START_WITH (plugin, "image/x-xcf", GST_RANK_SECONDARY,
@@ -1496,14 +1966,31 @@ plugin_init (GstPlugin * plugin)
       ogmvideo_type_find, NULL, OGMVIDEO_CAPS, NULL);
   TYPE_FIND_REGISTER (plugin, "application/x-ogm-audio", GST_RANK_PRIMARY,
       ogmaudio_type_find, NULL, OGMAUDIO_CAPS, NULL);
+  TYPE_FIND_REGISTER (plugin, "application/x-ogm-text", GST_RANK_PRIMARY,
+      ogmtext_type_find, NULL, OGMTEXT_CAPS, NULL);
   TYPE_FIND_REGISTER (plugin, "audio/x-speex", GST_RANK_PRIMARY,
       speex_type_find, NULL, SPEEX_CAPS, NULL);
+  TYPE_FIND_REGISTER (plugin, "application/x-ogg-skeleton", GST_RANK_PRIMARY,
+      oggskel_type_find, NULL, OGG_SKELETON_CAPS, NULL);
+  TYPE_FIND_REGISTER (plugin, "text/x-cmml", GST_RANK_PRIMARY,
+      cmml_type_find, NULL, CMML_CAPS, NULL);
   TYPE_FIND_REGISTER (plugin, "audio/x-m4a", GST_RANK_PRIMARY, m4a_type_find,
       m4a_exts, M4A_CAPS, NULL);
+  TYPE_FIND_REGISTER (plugin, "application/x-3gp", GST_RANK_PRIMARY,
+      q3gp_type_find, q3gp_exts, Q3GP_CAPS, NULL);
   TYPE_FIND_REGISTER_START_WITH (plugin, "application/x-executable",
       GST_RANK_MARGINAL, NULL, "\177ELF", 4, GST_TYPE_FIND_MAXIMUM);
+  TYPE_FIND_REGISTER_START_WITH (plugin, "audio/x-midi-file",
+      GST_RANK_MARGINAL, NULL, "MThd", 4, GST_TYPE_FIND_MAXIMUM);
   TYPE_FIND_REGISTER (plugin, "adts_mpeg_stream", GST_RANK_SECONDARY,
       aac_type_find, aac_exts, AAC_CAPS, NULL);
+  TYPE_FIND_REGISTER_START_WITH (plugin, "audio/x-spc", GST_RANK_SECONDARY,
+      spc_exts, "SNES-SPC700 Sound File Data", 27, GST_TYPE_FIND_MAXIMUM);
+  TYPE_FIND_REGISTER (plugin, "audio/x-wavpack", GST_RANK_SECONDARY,
+      wavpack_type_find, wavpack_exts, WAVPACK_CAPS, NULL);
+  TYPE_FIND_REGISTER (plugin, "audio/x-wavpack-correction", GST_RANK_SECONDARY,
+      wavpack_type_find, wavpack_correction_exts,
+      WAVPACK_CORRECTION_CAPS, NULL);
 
   return TRUE;
 }
