@@ -235,6 +235,9 @@ gst_video_crop_set_property (GObject * object, guint prop_id,
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
   }
+
+  if (gst_pad_is_negotiated (video_crop->srcpad))
+    video_crop->renegotiate_src_caps = TRUE;
 }
 static void
 gst_video_crop_get_property (GObject * object, guint prop_id, GValue * value,
@@ -263,9 +266,6 @@ gst_video_crop_get_property (GObject * object, guint prop_id, GValue * value,
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
   }
-
-  if (gst_pad_is_negotiated (video_crop->srcpad))
-    video_crop->renegotiate_src_caps = TRUE;
 }
 
 static void
@@ -523,7 +523,8 @@ gst_video_crop_chain (GstPad * pad, GstData * _data)
 
   outbuf = gst_pad_alloc_buffer (video_crop->srcpad, GST_BUFFER_OFFSET (buffer),
       GST_VIDEO_I420_SIZE (new_width, new_height));
-  GST_BUFFER_TIMESTAMP (outbuf) = GST_BUFFER_TIMESTAMP (buffer);
+
+  gst_buffer_stamp (outbuf, buffer);
 
   gst_video_crop_i420 (video_crop, buffer, outbuf);
   gst_buffer_unref (buffer);
