@@ -272,6 +272,7 @@ gst_videomixer_pad_sinkconnect (GstPad * pad, const GstCaps * vscaps)
   GstVideoMixer *mix;
   GstVideoMixerPad *mixpad;
   GstStructure *structure;
+  gint in_width, in_height;
 
   mix = GST_VIDEO_MIXER (gst_pad_get_parent (pad));
   mixpad = GST_VIDEO_MIXER_PAD (pad);
@@ -280,9 +281,14 @@ gst_videomixer_pad_sinkconnect (GstPad * pad, const GstCaps * vscaps)
 
   structure = gst_caps_get_structure (vscaps, 0);
 
-  gst_structure_get_int (structure, "width", &mixpad->in_width);
-  gst_structure_get_int (structure, "height", &mixpad->in_height);
-  gst_structure_get_double (structure, "framerate", &mixpad->in_framerate);
+  if (!gst_structure_get_int (structure, "width", &in_width)
+      || !gst_structure_get_int (structure, "height", &in_height)
+      || !gst_structure_get_double (structure, "framerate",
+          &mixpad->in_framerate))
+    return GST_PAD_LINK_REFUSED;
+
+  mixpad->in_width = in_width;
+  mixpad->in_height = in_height;
 
   mixpad->xpos = 0;
   mixpad->ypos = 0;
@@ -380,7 +386,7 @@ static GstStaticPadTemplate src_factory = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("video/x-raw-yuv,"
-        "format = (fourcc) I420,"
+        "format = (fourcc) AYUV,"
         "width = (int) [ 16, 4096 ],"
         "height = (int) [ 16, 4096 ]," "framerate = (double) [ 0, max ]")
     );
@@ -746,26 +752,25 @@ gst_videomixer_handle_src_event (GstPad * pad, GstEvent * event)
 #define BLEND_MODE BLEND_HARDLIGHT
 #endif
 
+#define ROUND_UP_2(x) (((x) + 1) & ~1)
+#define ROUND_UP_4(x) (((x) + 3) & ~3)
+#define ROUND_UP_8(x) (((x) + 7) & ~7)
+
 /* note that this function does packing conversion and blending at the
  * same time */
 static void
-gst_videomixer_blend_ayuv_i420 (guint8 * src, gint xpos, gint ypos,
+gst_videomixer_blend_ayuv_ayuv (guint8 * src, gint xpos, gint ypos,
     gint src_width, gint src_height, gdouble src_alpha,
     guint8 * dest, gint dest_width, gint dest_height)
 {
-  gint dest_size;
   gint alpha, b_alpha;
-  guint8 *destY1, *destY2, *destU, *destV;
-  gint accumU;
-  gint accumV;
   gint i, j;
-  gint src_stride;
-  gint src_add, destY_add, destC_add;
-  guint8 *src1, *src2;
+  gint src_stride, dest_stride;
+  gint src_add, dest_add;
   gint Y, U, V;
 
-  src_stride = src_width * 4;
-  dest_size = dest_width * dest_height;
+  src_stride = ROUND_UP_2 (src_width) * 4;
+  dest_stride = ROUND_UP_2 (dest_width) * 4;
 
   b_alpha = (gint) (src_alpha * 255);
 
@@ -788,63 +793,27 @@ gst_videomixer_blend_ayuv_i420 (guint8 * src, gint xpos, gint ypos,
     src_height = dest_height - ypos;
   }
 
-  src_add = 2 * src_stride - (4 * src_width);
-  destY_add = 2 * dest_width - (src_width);
-  destC_add = dest_width / 2 - (src_width / 2);
+  src_add = src_stride - (4 * ROUND_UP_2 (src_width));
+  dest_add = dest_stride - (4 * ROUND_UP_2 (src_width));
 
-  destY1 = dest + xpos + (ypos * dest_width);
-  destY2 = destY1 + dest_width;
-  destU = dest + dest_size + xpos / 2 + (ypos / 2 * dest_width / 2);
-  destV = destU + dest_size / 4;
-
-  src1 = src;
-  src2 = src + src_stride;
+  dest = dest + 4 * xpos + (ypos * dest_stride);
 
   /* we convert a square of 2x2 samples to generate 4 Luma and 2 chroma samples */
-  for (i = 0; i < src_height / 2; i++) {
-    for (j = 0; j < src_width / 2; j++) {
-      alpha = (src1[0] * b_alpha) >> 8;
-      BLEND_MODE (destY1[0], destU[0], destV[0], src1[1], src1[2], src1[3],
+  for (i = 0; i < ROUND_UP_2 (src_height); i++) {
+    for (j = 0; j < ROUND_UP_2 (src_width); j++) {
+      alpha = (src[0] * b_alpha) >> 8;
+      BLEND_MODE (dest[1], dest[2], dest[3], src[1], src[2], src[3],
           alpha, Y, U, V);
-      destY1[0] = Y;
-      accumU = U;
-      accumV = V;
-      alpha = (src1[4] * b_alpha) >> 8;
-      BLEND_MODE (destY1[1], destU[0], destV[0], src1[5], src1[6], src1[7],
-          alpha, Y, U, V);
-      destY1[1] = Y;
-      accumU += U;
-      accumV += V;
-      alpha = (src2[0] * b_alpha) >> 8;
-      BLEND_MODE (destY2[0], destU[0], destV[0], src2[1], src2[2], src2[3],
-          alpha, Y, U, V);
-      destY2[0] = Y;
-      accumU += U;
-      accumV += V;
-      alpha = (src2[4] * b_alpha) >> 8;
-      BLEND_MODE (destY2[1], destU[0], destV[0], src2[5], src2[6], src2[7],
-          alpha, Y, U, V);
-      destY2[1] = Y;
-      accumU += U;
-      accumV += V;
+      dest[0] = 0xff;
+      dest[1] = Y;
+      dest[2] = U;
+      dest[3] = V;
 
-      /* take the average of the 4 chroma samples to get the final value */
-      destU[0] = accumU / 4;
-      destV[0] = accumV / 4;
-
-      src1 += 8;
-      src2 += 8;
-      destY1 += 2;
-      destY2 += 2;
-      destU += 1;
-      destV += 1;
+      src += 4;
+      dest += 4;
     }
-    src1 += src_add;
-    src2 += src_add;
-    destY1 += destY_add;
-    destY2 += destY_add;
-    destU += destC_add;
-    destV += destC_add;
+    src += src_add;
+    dest += dest_add;
   }
 }
 
@@ -868,27 +837,39 @@ gst_videomixer_sort_pads (GstVideoMixer * mix)
 static void
 gst_videomixer_fill_checker (guint8 * dest, gint width, gint height)
 {
-  gint size = width * height;
+  gint stride;
   gint i, j;
   static int tab[] = { 80, 160, 80, 160 };
 
+  stride = ROUND_UP_2 (width);
+
   for (i = 0; i < height; i++) {
-    for (j = 0; j < width; j++) {
+    for (j = 0; j < stride; j++) {
+      *dest++ = 0xff;
       *dest++ = tab[((i & 0x8) >> 3) + ((j & 0x8) >> 3)];
+      *dest++ = 128;
+      *dest++ = 128;
     }
   }
-  memset (dest, 128, size / 2);
 }
 
 static void
 gst_videomixer_fill_color (guint8 * dest, gint width, gint height,
     gint colY, gint colU, gint colV)
 {
-  gint size = width * height;
+  gint stride;
+  gint i, j;
 
-  memset (dest, colY, size);
-  memset (dest + size, colU, size / 4);
-  memset (dest + size + size / 4, colV, size / 4);
+  stride = ROUND_UP_2 (width);
+
+  for (i = 0; i < height; i++) {
+    for (j = 0; j < stride; j++) {
+      *dest++ = 0xff;
+      *dest++ = colY;
+      *dest++ = colU;
+      *dest++ = colV;
+    }
+  }
 }
 
 /* try to get a buffer on all pads. As long as the queued value is
@@ -986,11 +967,15 @@ gst_videomixer_blend_buffers (GstVideoMixer * mix, GstBuffer * outbuf)
     walk = g_slist_next (walk);
 
     if (pad->buffer != NULL) {
-      gst_videomixer_blend_ayuv_i420 (GST_BUFFER_DATA (pad->buffer),
+      gst_videomixer_blend_ayuv_ayuv (GST_BUFFER_DATA (pad->buffer),
           pad->xpos, pad->ypos,
           pad->in_width, pad->in_height,
           pad->alpha,
           GST_BUFFER_DATA (outbuf), mix->out_width, mix->out_height);
+      if (pad == mix->master) {
+        GST_BUFFER_TIMESTAMP (outbuf) = GST_BUFFER_TIMESTAMP (pad->buffer);
+        GST_BUFFER_DURATION (outbuf) = GST_BUFFER_DURATION (pad->buffer);
+      }
     }
   }
 }
@@ -1064,7 +1049,7 @@ gst_videomixer_loop (GstElement * element)
     newcaps =
         gst_caps_copy (gst_pad_get_negotiated_caps (GST_PAD (mix->master)));
     gst_caps_set_simple (newcaps, "format", GST_TYPE_FOURCC,
-        GST_STR_FOURCC ("I420"), "width", G_TYPE_INT, new_width, "height",
+        GST_STR_FOURCC ("AYUV"), "width", G_TYPE_INT, new_width, "height",
         G_TYPE_INT, new_height, NULL);
 
     if (GST_PAD_LINK_FAILED (gst_pad_try_set_caps (mix->srcpad, newcaps))) {
@@ -1076,7 +1061,8 @@ gst_videomixer_loop (GstElement * element)
     mix->out_height = new_height;
   }
 
-  outsize = 3 * (mix->out_width * mix->out_height) / 2;
+  outsize = ROUND_UP_2 (mix->out_width) * ROUND_UP_2 (mix->out_height) * 4;
+
   outbuf = gst_pad_alloc_buffer (mix->srcpad, GST_BUFFER_OFFSET_NONE, outsize);
   switch (mix->background) {
     case VIDEO_MIXER_BACKGROUND_CHECKER:
