@@ -48,6 +48,7 @@ MotifWmHints, MwmHints;
 static void gst_ximagesink_buffer_free (GstBuffer * buffer);
 static void gst_ximagesink_ximage_destroy (GstXImageSink * ximagesink,
     GstXImage * ximage);
+static void gst_ximagesink_send_pending_navigation (GstXImageSink * ximagesink);
 
 /* ElementFactory information */
 static GstElementDetails gst_ximagesink_details =
@@ -136,7 +137,7 @@ gst_ximagesink_check_xshm_calls (GstXContext * xcontext)
     goto beach;
   }
 
-  ximage->SHMInfo.shmaddr = shmat (ximage->SHMInfo.shmid, 0, 0);
+  ximage->SHMInfo.shmaddr = shmat (ximage->SHMInfo.shmid, NULL, 0);
   if (ximage->SHMInfo.shmaddr == ((void *) -1)) {
     GST_WARNING ("Failed to shmat: %s", g_strerror (errno));
     goto beach;
@@ -156,7 +157,7 @@ gst_ximagesink_check_xshm_calls (GstXContext * xcontext)
   XSync (xcontext->disp, FALSE);
 
   shmdt (ximage->SHMInfo.shmaddr);
-  shmctl (ximage->SHMInfo.shmid, IPC_RMID, 0);
+  shmctl (ximage->SHMInfo.shmid, IPC_RMID, NULL);
 
   /* To be sure, reset the SHMInfo entry */
   ximage->SHMInfo.shmaddr = ((void *) -1);
@@ -218,7 +219,7 @@ gst_ximagesink_ximage_new (GstXImageSink * ximagesink, gint width, gint height)
       goto beach;
     }
 
-    ximage->SHMInfo.shmaddr = shmat (ximage->SHMInfo.shmid, 0, 0);
+    ximage->SHMInfo.shmaddr = shmat (ximage->SHMInfo.shmid, NULL, 0);
     if (ximage->SHMInfo.shmaddr == ((void *) -1)) {
       GST_ELEMENT_ERROR (ximagesink, RESOURCE, WRITE, (NULL),
           ("Failed to shmat: %s", g_strerror (errno)));
@@ -288,7 +289,7 @@ gst_ximagesink_ximage_destroy (GstXImageSink * ximagesink, GstXImage * ximage)
       shmdt (ximage->SHMInfo.shmaddr);
     }
     if (ximage->SHMInfo.shmid > 0)
-      shmctl (ximage->SHMInfo.shmid, IPC_RMID, 0);
+      shmctl (ximage->SHMInfo.shmid, IPC_RMID, NULL);
     if (ximage->ximage)
       XDestroyImage (ximage->ximage);
 
@@ -345,6 +346,7 @@ gst_ximagesink_ximage_put (GstXImageSink * ximagesink, GstXImage * ximage)
 
   XSync (ximagesink->xcontext->disp, FALSE);
 
+  ximagesink->need_expose = FALSE;
   g_mutex_unlock (ximagesink->x_lock);
 }
 
@@ -389,6 +391,7 @@ static GstXWindow *
 gst_ximagesink_xwindow_new (GstXImageSink * ximagesink, gint width, gint height)
 {
   GstXWindow *xwindow = NULL;
+  XGCValues values;
 
   g_return_val_if_fail (GST_IS_XIMAGESINK (ximagesink), NULL);
 
@@ -408,7 +411,8 @@ gst_ximagesink_xwindow_new (GstXImageSink * ximagesink, gint width, gint height)
       StructureNotifyMask | PointerMotionMask | KeyPressMask |
       KeyReleaseMask | ButtonPressMask | ButtonReleaseMask);
 
-  xwindow->gc = XCreateGC (ximagesink->xcontext->disp, xwindow->win, 0, NULL);
+  xwindow->gc = XCreateGC (ximagesink->xcontext->disp, xwindow->win,
+      0, &values);
 
   XMapRaised (ximagesink->xcontext->disp, xwindow->win);
 
@@ -856,6 +860,7 @@ static void
 gst_ximagesink_xcontext_clear (GstXImageSink * ximagesink)
 {
   g_return_if_fail (GST_IS_XIMAGESINK (ximagesink));
+  g_return_if_fail (ximagesink->xcontext != NULL);
 
   gst_caps_free (ximagesink->xcontext->caps);
   g_free (ximagesink->xcontext->par);
@@ -868,6 +873,7 @@ gst_ximagesink_xcontext_clear (GstXImageSink * ximagesink)
 
   g_mutex_unlock (ximagesink->x_lock);
 
+  g_free (ximagesink->xcontext);
   ximagesink->xcontext = NULL;
 }
 
@@ -974,6 +980,8 @@ gst_ximagesink_sink_link (GstPad * pad, const GstCaps * caps)
   if (!ret)
     return GST_PAD_LINK_REFUSED;
 
+  g_mutex_lock (ximagesink->stream_lock);
+
   /* if the caps contain pixel-aspect-ratio, they have to match ours,
    * otherwise linking should fail */
   par = gst_structure_get_value (structure, "pixel-aspect-ratio");
@@ -1006,6 +1014,8 @@ gst_ximagesink_sink_link (GstPad * pad, const GstCaps * caps)
     ximagesink->ximage = NULL;
   }
 
+  g_mutex_unlock (ximagesink->stream_lock);
+
   gst_x_overlay_got_desired_size (GST_X_OVERLAY (ximagesink),
       GST_VIDEOSINK_WIDTH (ximagesink), GST_VIDEOSINK_HEIGHT (ximagesink));
 
@@ -1029,11 +1039,11 @@ gst_ximagesink_change_state (GstElement * element)
       /* call XSynchronize with the current value of synchronous */
       GST_DEBUG_OBJECT (ximagesink, "XSynchronize called with %s",
           ximagesink->synchronous ? "TRUE" : "FALSE");
+      g_mutex_lock (ximagesink->x_lock);
       XSynchronize (ximagesink->xcontext->disp, ximagesink->synchronous);
+      g_mutex_unlock (ximagesink->x_lock);
       break;
     case GST_STATE_READY_TO_PAUSED:
-      if (ximagesink->xwindow)
-        gst_ximagesink_xwindow_clear (ximagesink, ximagesink->xwindow);
       ximagesink->time = 0;
       break;
     case GST_STATE_PAUSED_TO_PLAYING:
@@ -1041,12 +1051,20 @@ gst_ximagesink_change_state (GstElement * element)
     case GST_STATE_PLAYING_TO_PAUSED:
       break;
     case GST_STATE_PAUSED_TO_READY:
+      g_mutex_lock (ximagesink->stream_lock);
+      if (ximagesink->xwindow)
+        gst_ximagesink_xwindow_clear (ximagesink, ximagesink->xwindow);
       ximagesink->framerate = 0;
       ximagesink->sw_scaling_failed = FALSE;
       GST_VIDEOSINK_WIDTH (ximagesink) = 0;
       GST_VIDEOSINK_HEIGHT (ximagesink) = 0;
+      g_mutex_unlock (ximagesink->stream_lock);
       break;
     case GST_STATE_READY_TO_NULL:
+      /* We are cleaning our resources here, yes i know chain is not running
+         but the interface can be called to set a window from a different thread
+         and that would crash */
+      g_mutex_lock (ximagesink->stream_lock);
       if (ximagesink->ximage) {
         gst_ximagesink_ximage_destroy (ximagesink, ximagesink->ximage);
         ximagesink->ximage = NULL;
@@ -1064,6 +1082,7 @@ gst_ximagesink_change_state (GstElement * element)
         gst_ximagesink_xcontext_clear (ximagesink);
         ximagesink->xcontext = NULL;
       }
+      g_mutex_unlock (ximagesink->stream_lock);
       break;
   }
 
@@ -1073,19 +1092,72 @@ gst_ximagesink_change_state (GstElement * element)
   return GST_STATE_SUCCESS;
 }
 
+static void
+gst_ximagesink_handle_event (GstXImageSink * ximagesink, GstEvent * event,
+    GstPad * pad)
+{
+  switch (GST_EVENT_TYPE (event)) {
+    case GST_EVENT_FILLER:
+    {
+      GstClockTime ts = GST_EVENT_TIMESTAMP (event);
+      GstClockTime dur = gst_event_filler_get_duration (event);
+      GstClockTimeDiff step;
+
+      if (!GST_CLOCK_TIME_IS_VALID (ts)) {
+        ts = ximagesink->time;
+      }
+      if (GST_CLOCK_TIME_IS_VALID (dur)) {
+        ts += dur;
+      }
+
+      gst_event_unref (event);
+      event = NULL;
+
+      if (ximagesink->framerate > 0)
+        step = GST_SECOND / ximagesink->framerate;
+      else
+        step = GST_CLOCK_DIFF (ts, ximagesink->time);
+
+      /*
+       * Advance the clock to the timestamp of the event, in steps
+       * at the framerate so we send navigation events and redraw nicely.
+       */
+      if (step > 0 && GST_VIDEOSINK_CLOCK (ximagesink)) {
+        while (ximagesink->time < ts) {
+          GstClockTime next_ts = MIN (ximagesink->time + step, ts);
+
+          if (next_ts > ximagesink->time) {
+            gst_element_wait (GST_ELEMENT (ximagesink), next_ts);
+          }
+          ximagesink->time = next_ts;
+
+          g_mutex_lock (ximagesink->stream_lock);
+          gst_ximagesink_handle_xevents (ximagesink, pad);
+          g_mutex_unlock (ximagesink->stream_lock);
+          gst_ximagesink_send_pending_navigation (ximagesink);
+        }
+      }
+
+      break;
+    }
+    default:
+      gst_pad_event_default (pad, event);
+  }
+}
+
 static void
 gst_ximagesink_chain (GstPad * pad, GstData * data)
 {
-  GstBuffer *buf = GST_BUFFER (data);
+  GstBuffer *buf;
   GstXImageSink *ximagesink;
 
   g_return_if_fail (GST_IS_PAD (pad));
-  g_return_if_fail (buf != NULL);
+  g_return_if_fail (data != NULL);
 
   ximagesink = GST_XIMAGESINK (gst_pad_get_parent (pad));
 
   if (GST_IS_EVENT (data)) {
-    gst_pad_event_default (pad, GST_EVENT (data));
+    gst_ximagesink_handle_event (ximagesink, GST_EVENT (data), pad);
     return;
   }
 
@@ -1101,6 +1173,8 @@ gst_ximagesink_chain (GstPad * pad, GstData * data)
     gst_element_wait (GST_ELEMENT (ximagesink), ximagesink->time);
   }
 
+  g_mutex_lock (ximagesink->stream_lock);
+
   /* If this buffer has been allocated using our buffer management we simply
      put the ximage which is in the PRIVATE pointer */
   if (GST_BUFFER_FREE_DATA_FUNC (buf) == gst_ximagesink_buffer_free) {
@@ -1137,6 +1211,9 @@ gst_ximagesink_chain (GstPad * pad, GstData * data)
   gst_buffer_unref (buf);
 
   gst_ximagesink_handle_xevents (ximagesink, pad);
+
+  g_mutex_unlock (ximagesink->stream_lock);
+  gst_ximagesink_send_pending_navigation (ximagesink);
 }
 
 /* Buffer management */
@@ -1213,7 +1290,7 @@ gst_ximagesink_buffer_alloc (GstPad * pad, guint64 offset, guint size)
     /* Storing some pointers in the buffer */
     GST_BUFFER_PRIVATE (buffer) = ximage;
 
-    GST_BUFFER_DATA (buffer) = ximage->ximage->data;
+    GST_BUFFER_DATA (buffer) = (guint8 *) ximage->ximage->data;
     GST_BUFFER_FREE_DATA_FUNC (buffer) = gst_ximagesink_buffer_free;
     GST_BUFFER_SIZE (buffer) = ximage->size;
     return buffer;
@@ -1236,37 +1313,82 @@ gst_ximagesink_interface_init (GstImplementsInterfaceClass * klass)
   klass->supported = gst_ximagesink_interface_supported;
 }
 
+/*
+ * This function is called with the stream-lock held
+ */
+static void
+gst_ximagesink_send_pending_navigation (GstXImageSink * ximagesink)
+{
+  GSList *cur;
+  GSList *pend_events;
+
+  g_mutex_lock (ximagesink->nav_lock);
+  pend_events = ximagesink->pend_nav_events;
+  ximagesink->pend_nav_events = NULL;
+  g_mutex_unlock (ximagesink->nav_lock);
+
+  cur = pend_events;
+  while (cur) {
+    GstEvent *event = cur->data;
+    GstStructure *structure;
+    double x, y;
+    gint x_offset, y_offset;
+
+    if (event) {
+      structure = event->event_data.structure.structure;
+
+      if (!GST_PAD_PEER (GST_VIDEOSINK_PAD (ximagesink))) {
+        gst_event_unref (event);
+        cur = g_slist_next (cur);
+        continue;
+      }
+
+      /* We are not converting the pointer coordinates as there's no hardware
+         scaling done here. The only possible scaling is done by videoscale and
+         videoscale will have to catch those events and tranform the coordinates
+         to match the applied scaling. So here we just add the offset if the image
+         is centered in the window.  */
+
+      g_mutex_lock (ximagesink->stream_lock);
+      x_offset = ximagesink->xwindow->width - GST_VIDEOSINK_WIDTH (ximagesink);
+      y_offset =
+          ximagesink->xwindow->height - GST_VIDEOSINK_HEIGHT (ximagesink);
+
+      if (gst_structure_get_double (structure, "pointer_x", &x)) {
+        x -= x_offset / 2;
+        gst_structure_set (structure, "pointer_x", G_TYPE_DOUBLE, x, NULL);
+      }
+      if (gst_structure_get_double (structure, "pointer_y", &y)) {
+        y -= y_offset / 2;
+        gst_structure_set (structure, "pointer_y", G_TYPE_DOUBLE, y, NULL);
+      }
+      g_mutex_unlock (ximagesink->stream_lock);
+
+      GST_DATA_FLAG_UNSET (event, GST_EVENT_COMMON_FLAG_NEED_RESPONSE);
+
+      gst_pad_send_event (gst_pad_get_peer (GST_VIDEOSINK_PAD (ximagesink)),
+          event);
+    }
+    cur = g_slist_next (cur);
+  }
+
+  g_slist_free (pend_events);
+}
+
 static void
 gst_ximagesink_navigation_send_event (GstNavigation * navigation,
     GstStructure * structure)
 {
   GstXImageSink *ximagesink = GST_XIMAGESINK (navigation);
   GstEvent *event;
-  gint x_offset, y_offset;
-  double x, y;
 
   event = gst_event_new (GST_EVENT_NAVIGATION);
   event->event_data.structure.structure = structure;
 
-  /* We are not converting the pointer coordinates as there's no hardware
-     scaling done here. The only possible scaling is done by videoscale and
-     videoscale will have to catch those events and tranform the coordinates
-     to match the applied scaling. So here we just add the offset if the image
-     is centered in the window.  */
-
-  x_offset = ximagesink->xwindow->width - GST_VIDEOSINK_WIDTH (ximagesink);
-  y_offset = ximagesink->xwindow->height - GST_VIDEOSINK_HEIGHT (ximagesink);
-
-  if (gst_structure_get_double (structure, "pointer_x", &x)) {
-    x += x_offset;
-    gst_structure_set (structure, "pointer_x", G_TYPE_DOUBLE, x, NULL);
-  }
-  if (gst_structure_get_double (structure, "pointer_y", &y)) {
-    y += y_offset;
-    gst_structure_set (structure, "pointer_y", G_TYPE_DOUBLE, y, NULL);
-  }
-
-  gst_pad_send_event (gst_pad_get_peer (GST_VIDEOSINK_PAD (ximagesink)), event);
+  g_mutex_lock (ximagesink->nav_lock);
+  ximagesink->pend_nav_events =
+      g_slist_prepend (ximagesink->pend_nav_events, event);
+  g_mutex_unlock (ximagesink->nav_lock);
 }
 
 static void
@@ -1297,6 +1419,11 @@ gst_ximagesink_set_xwindow_id (GstXOverlay * overlay, XID xwindow_id)
     return;
   }
 
+  /* We acquire the stream lock while setting this window in the element.
+     We are basically cleaning tons of stuff replacing the old window, putting
+     images while we do that would surely crash */
+  g_mutex_lock (ximagesink->stream_lock);
+
   /* Clear image pool as the images are unusable anyway */
   gst_ximagesink_imagepool_clear (ximagesink);
 
@@ -1380,6 +1507,8 @@ gst_ximagesink_set_xwindow_id (GstXOverlay * overlay, XID xwindow_id)
 
   if (xwindow)
     ximagesink->xwindow = xwindow;
+
+  g_mutex_unlock (ximagesink->stream_lock);
 }
 
 static void
@@ -1400,6 +1529,16 @@ gst_ximagesink_expose (GstXOverlay * overlay)
   if (!ximagesink->xwindow)
     return;
 
+  /* Set the need_expose flag before we wait for the lock. 
+   * If it's clear by the time we get to put the image,
+   * the stream beat us to it so don't bother
+   */
+  ximagesink->need_expose = TRUE;
+
+  /* We don't want chain to iterate while we do that. We might act on random
+     cur_image and different geometry */
+  g_mutex_lock (ximagesink->stream_lock);
+
   gst_ximagesink_xwindow_update_geometry (ximagesink, ximagesink->xwindow);
 
   /* We don't act on internal window from outside that could cause some thread
@@ -1409,8 +1548,10 @@ gst_ximagesink_expose (GstXOverlay * overlay)
 
   gst_ximagesink_xwindow_clear (ximagesink, ximagesink->xwindow);
 
-  if (ximagesink->cur_image)
+  if (ximagesink->need_expose && ximagesink->cur_image)
     gst_ximagesink_ximage_put (ximagesink, ximagesink->cur_image);
+
+  g_mutex_unlock (ximagesink->stream_lock);
 }
 
 static void
@@ -1446,7 +1587,9 @@ gst_ximagesink_set_property (GObject * object, guint prop_id,
       if (ximagesink->xcontext) {
         GST_DEBUG_OBJECT (ximagesink, "XSynchronize called with %s",
             ximagesink->synchronous ? "TRUE" : "FALSE");
+        g_mutex_lock (ximagesink->x_lock);
         XSynchronize (ximagesink->xcontext->disp, ximagesink->synchronous);
+        g_mutex_unlock (ximagesink->x_lock);
       }
       break;
     case ARG_PIXEL_ASPECT_RATIO:
@@ -1506,8 +1649,35 @@ gst_ximagesink_finalize (GObject * object)
     ximagesink->display_name = NULL;
   }
 
-  g_mutex_free (ximagesink->x_lock);
-  g_mutex_free (ximagesink->pool_lock);
+  if (ximagesink->par) {
+    g_free (ximagesink->par);
+    ximagesink->par = NULL;
+  }
+  if (ximagesink->x_lock) {
+    g_mutex_free (ximagesink->x_lock);
+    ximagesink->x_lock = NULL;
+  }
+  if (ximagesink->stream_lock) {
+    g_mutex_free (ximagesink->stream_lock);
+    ximagesink->stream_lock = NULL;
+  }
+  if (ximagesink->pool_lock) {
+    g_mutex_free (ximagesink->pool_lock);
+    ximagesink->pool_lock = NULL;
+  }
+  if (ximagesink->nav_lock) {
+    g_mutex_free (ximagesink->nav_lock);
+    ximagesink->nav_lock = NULL;
+  }
+
+  while (ximagesink->pend_nav_events) {
+    GstEvent *event = ximagesink->pend_nav_events->data;
+
+    ximagesink->pend_nav_events =
+        g_slist_delete_link (ximagesink->pend_nav_events,
+        ximagesink->pend_nav_events);
+    gst_event_unref (event);
+  }
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -1542,16 +1712,20 @@ gst_ximagesink_init (GstXImageSink * ximagesink)
   ximagesink->framerate = 0;
 
   ximagesink->x_lock = g_mutex_new ();
-
+  ximagesink->stream_lock = g_mutex_new ();
 
   ximagesink->image_pool = NULL;
   ximagesink->pool_lock = g_mutex_new ();
 
   ximagesink->sw_scaling_failed = FALSE;
   ximagesink->synchronous = FALSE;
+  ximagesink->need_expose = FALSE;
 
   ximagesink->par = NULL;
 
+  ximagesink->nav_lock = g_mutex_new ();
+  ximagesink->pend_nav_events = NULL;
+
   GST_FLAG_SET (ximagesink, GST_ELEMENT_THREAD_SUGGESTED);
   GST_FLAG_SET (ximagesink, GST_ELEMENT_EVENT_AWARE);
 }
@@ -1594,6 +1768,9 @@ gst_ximagesink_class_init (GstXImageSinkClass * klass)
   gobject_class->get_property = gst_ximagesink_get_property;
 
   gstelement_class->change_state = gst_ximagesink_change_state;
+
+  GST_DEBUG_CATEGORY_INIT (gst_debug_ximagesink, "ximagesink", 0,
+      "ximagesink element");
 }
 
 /* ============================================================= */
@@ -1654,26 +1831,3 @@ gst_ximagesink_get_type (void)
 
   return ximagesink_type;
 }
-
-static gboolean
-plugin_init (GstPlugin * plugin)
-{
-  /* Loading the library containing GstVideoSink, our parent object */
-  if (!gst_library_load ("gstvideo"))
-    return FALSE;
-
-  if (!gst_element_register (plugin, "ximagesink",
-          GST_RANK_SECONDARY, GST_TYPE_XIMAGESINK))
-    return FALSE;
-
-  GST_DEBUG_CATEGORY_INIT (gst_debug_ximagesink, "ximagesink", 0,
-      "ximagesink element");
-
-  return TRUE;
-}
-
-GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
-    GST_VERSION_MINOR,
-    "ximagesink",
-    "XFree86 video output plugin based on standard Xlib calls",
-    plugin_init, VERSION, GST_LICENSE, GST_PACKAGE, GST_ORIGIN)
